@@ -9,14 +9,14 @@ morphological measurements.
 # Affiliated packages may add whatever they like to this file, but
 # should keep this content at the top.
 # ----------------------------------------------------------------------------
-from ._astropy_init import *
+from ._astropy_init import *       # noqa
 # ----------------------------------------------------------------------------
 
-if not _ASTROPY_SETUP_:
-    from .aperture import *
-    from .background import *
-    from .centroids import *
-    from .detection import *
-    from .morphology import *
-    from .psf import *
-    from .segmentation import *
+if not _ASTROPY_SETUP_:            # noqa
+    from .aperture import *        # noqa
+    from .background import *      # noqa
+    from .centroids import *       # noqa
+    from .detection import *       # noqa
+    from .morphology import *      # noqa
+    from .psf import *             # noqa
+    from .segmentation import *    # noqa
