@@ -6,8 +6,9 @@ import copy
 import warnings
 from collections import OrderedDict
 
+import six
 import numpy as np
-from astropy.extern import six
+from astropy.coordinates import SkyCoord
 from astropy.io import fits
 from astropy.nddata import support_nddata
 from astropy.table import QTable
@@ -15,81 +16,14 @@ import astropy.units as u
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils.misc import InheritDocstrings
 from astropy.wcs import WCS
+from astropy.wcs.utils import (skycoord_to_pixel, pixel_to_skycoord,
+                               wcs_to_celestial_frame)
+
+from ..utils import get_version_info
+from ..utils.wcs_helpers import pixel_scale_angle_at_skycoord
 
 
-__all__ = ['Aperture', 'SkyAperture', 'PixelAperture', 'ApertureMask',
-           'aperture_photometry']
-
-
-def _get_version_info():
-    from astropy import __version__
-    astropy_version = __version__
-
-    from photutils import __version__
-    photutils_version = __version__
-
-    return 'astropy: {0}, photutils: {1}'.format(astropy_version,
-                                                 photutils_version)
-
-
-def _sanitize_pixel_positions(positions):
-    if isinstance(positions, u.Quantity):
-        if positions.unit is u.pixel:
-            positions = np.atleast_2d(positions.value)
-        else:
-            raise u.UnitsError("positions should be in pixel units")
-    elif isinstance(positions, (list, tuple, np.ndarray)):
-        positions = np.atleast_2d(positions)
-        if positions.shape[1] != 2:
-            if positions.shape[0] == 2:
-                positions = np.transpose(positions)
-            else:
-                raise TypeError("List or array of (x, y) pixel coordinates "
-                                "is expected got '{0}'.".format(positions))
-    elif isinstance(positions, zip):
-        # This is needed for zip to work seamlessly in Python 3
-        positions = np.atleast_2d(list(positions))
-    else:
-        raise TypeError("List or array of (x, y) pixel coordinates "
-                        "is expected got '{0}'.".format(positions))
-
-    if positions.ndim > 2:
-        raise ValueError('{0}D position array not supported. Only 2D '
-                         'arrays supported.'.format(positions.ndim))
-
-    return positions
-
-
-def _translate_mask_method(method, subpixels):
-    if method == 'center':
-        use_exact = 0
-        subpixels = 1
-    elif method == 'subpixel':
-        use_exact = 0
-    elif method == 'exact':
-        use_exact = 1
-        subpixels = 1
-    else:
-        raise ValueError('"{0}" is not a valid method.'.format(method))
-
-    return use_exact, subpixels
-
-
-def _make_annulus_path(patch_inner, patch_outer):
-    import matplotlib.path as mpath
-
-    verts_inner = patch_inner.get_verts()
-    verts_outer = patch_outer.get_verts()
-    codes_inner = (np.ones(len(verts_inner), dtype=mpath.Path.code_type) *
-                   mpath.Path.LINETO)
-    codes_inner[0] = mpath.Path.MOVETO
-    codes_outer = (np.ones(len(verts_outer), dtype=mpath.Path.code_type) *
-                   mpath.Path.LINETO)
-    codes_outer[0] = mpath.Path.MOVETO
-    codes = np.concatenate((codes_inner, codes_outer))
-    verts = np.concatenate((verts_inner, verts_outer[::-1]))
-
-    return mpath.Path(verts, codes)
+__all__ = ['Aperture', 'SkyAperture', 'PixelAperture', 'aperture_photometry']
 
 
 class _ABCMetaAndInheritDocstrings(InheritDocstrings, abc.ABCMeta):
@@ -107,69 +41,130 @@ class Aperture(object):
             return 1
         return len(self.positions)
 
+    def _positions_str(self, prefix=None):
+        if isinstance(self, PixelAperture):
+            return np.array2string(self.positions, separator=', ',
+                                   prefix=prefix)
+        elif isinstance(self, SkyAperture):
+            return repr(self.positions)
+        else:
+            raise TypeError('Aperture must be a subclass of PixelAperture '
+                            'or SkyAperture')
 
-class SkyAperture(Aperture):
-    """
-    Abstract base class for 2D apertures defined in celestial coordinates.
-    """
+    def __repr__(self):
+        prefix = '<{0}('.format(self.__class__.__name__)
+        params = [self._positions_str(prefix)]
+        for param in self._params:
+            params.append('{0}={1}'.format(param, getattr(self, param)))
+        params = ', '.join(params)
 
-    @abc.abstractmethod
-    def to_pixel(self, wcs, mode='all'):
-        """
-        Convert the aperture to a `PixelAperture` object in pixel
-        coordinates.
+        return '{0}{1})>'.format(prefix, params)
 
-        Parameters
-        ----------
-        wcs : `~astropy.wcs.WCS`
-            The WCS transformation to use.
+    def __str__(self):
+        prefix = 'positions'
+        cls_info = [
+            ('Aperture', self.__class__.__name__),
+            (prefix, self._positions_str(prefix + ': '))]
+        if self._params is not None:
+            for param in self._params:
+                cls_info.append((param, getattr(self, param)))
+        fmt = ['{0}: {1}'.format(key, val) for key, val in cls_info]
 
-        mode : {'all', 'wcs'}, optional
-            Whether to do the transformation including distortions
-            (``'all'``; default) or only including only the core WCS
-            transformation (``'wcs'``).
-
-        Returns
-        -------
-        aperture : `PixelAperture` object
-            A `PixelAperture` object.
-        """
-
-        raise NotImplementedError('Needs to be implemented in a '
-                                  'SkyAperture subclass.')
+        return '\n'.join(fmt)
 
 
 class PixelAperture(Aperture):
     """
-    Abstract base class for 2D apertures defined in pixel coordinates.
-
-    Derived classes must define a ``_slices`` property, ``to_mask`` and
-    ``plot`` methods, and optionally an ``area`` method.
+    Abstract base class for apertures defined in pixel coordinates.
     """
 
+    @staticmethod
+    def _sanitize_positions(positions):
+        if isinstance(positions, u.Quantity):
+            if positions.unit is u.pixel:
+                positions = np.atleast_2d(positions.value)
+            else:
+                raise u.UnitsError('positions should be in pixel units')
+        elif isinstance(positions, (list, tuple, np.ndarray)):
+            positions = np.atleast_2d(positions)
+            if positions.shape[1] != 2:
+                if positions.shape[0] == 2:
+                    positions = np.transpose(positions)
+                else:
+                    raise TypeError('List or array of (x, y) pixel '
+                                    'coordinates is expected, got "{0}".'
+                                    .format(positions))
+        elif isinstance(positions, zip):
+            # This is needed for zip to work seamlessly in Python 3
+            positions = np.atleast_2d(list(positions))
+        else:
+            raise TypeError('List or array of (x, y) pixel coordinates '
+                            'is expected, got "{0}".'.format(positions))
+
+        if positions.ndim > 2:
+            raise ValueError('{0}D position array is not supported. Only 2D '
+                             'arrays are supported.'.format(positions.ndim))
+
+        return positions
+
+    @staticmethod
+    def _translate_mask_mode(mode, subpixels, rectangle=False):
+        if mode not in ('center', 'subpixel', 'exact'):
+            raise ValueError('Invalid mask mode: {0}'.format(mode))
+
+        if rectangle and mode == 'exact':
+            warnings.warn('The "exact" method is not yet implemented for '
+                          'rectangular apertures -- using "subpixel" method '
+                          'with "subpixels=32"', AstropyUserWarning)
+            mode = 'subpixel'
+            subpixels = 32
+
+        if mode == 'subpixels':
+            if not isinstance(subpixels, int) or subpixels <= 0:
+                raise ValueError('subpixels must be a strictly positive '
+                                 'integer'.format(subpixels))
+
+        if mode == 'center':
+            use_exact = 0
+            subpixels = 1
+        elif mode == 'subpixel':
+            use_exact = 0
+        elif mode == 'exact':
+            use_exact = 1
+            subpixels = 1
+
+        return use_exact, subpixels
+
     @abc.abstractproperty
-    def _slices(self):
-        """The minimal bounding box slices for the aperture(s)."""
+    def bounding_boxes(self):
+        """
+        A list of minimal bounding boxes (`~photutils.BoundingBox`), one
+        for each position, for the aperture.
+        """
 
         raise NotImplementedError('Needs to be implemented in a '
                                   'PixelAperture subclass.')
 
     @property
-    def _geom_slices(self):
+    def _centered_edges(self):
         """
-        A tuple of slices to be used by the low-level
-        `photutils.geometry` functions.
+        A list of ``(xmin, xmax, ymin, ymax)`` tuples, one for each
+        position, of the pixel edges after recentering the aperture at
+        the origin.
+
+        These pixel edges are used by the low-level `photutils.geometry`
+        functions.
         """
 
-        geom_slices = []
-        for _slice, position in zip(self._slices, self.positions):
-            x_min = _slice[1].start - position[0] - 0.5
-            x_max = _slice[1].stop - position[0] - 0.5
-            y_min = _slice[0].start - position[1] - 0.5
-            y_max = _slice[0].stop - position[1] - 0.5
-            geom_slices.append((slice(y_min, y_max), slice(x_min, x_max)))
+        edges = []
+        for position, bbox in zip(self.positions, self.bounding_boxes):
+            xmin = bbox.ixmin - 0.5 - position[0]
+            xmax = bbox.ixmax - 0.5 - position[0]
+            ymin = bbox.iymin - 0.5 - position[1]
+            ymax = bbox.iymax - 0.5 - position[1]
+            edges.append((xmin, xmax, ymin, ymax))
 
-        return geom_slices
+        return edges
 
     def area(self):
         """
@@ -238,8 +233,8 @@ class PixelAperture(Aperture):
     @abc.abstractmethod
     def to_mask(self, method='exact', subpixels=5):
         """
-        Return a list of `ApertureMask` objects, one for each aperture
-        position.
+        Return a list of `~photutils.ApertureMask` objects, one for each
+        aperture position.
 
         Parameters
         ----------
@@ -312,8 +307,8 @@ class PixelAperture(Aperture):
 
         return output
 
-    def do_photometry(self, data, error=None, pixelwise_error=True,
-                      mask=None, method='exact', subpixels=5, unit=None):
+    def do_photometry(self, data, error=None, mask=None, method='exact',
+                      subpixels=5, unit=None):
         """
         Perform aperture photometry on the input data.
 
@@ -330,12 +325,6 @@ class PixelAperture(Aperture):
             `~photutils.utils.calc_total_error`) .  ``error`` must have
             the same shape as the input ``data``.
 
-        pixelwise_error : bool, optional
-            If `True` (default), the photometric error is calculated
-            using the ``error`` values from each pixel within the
-            aperture.  If `False`, the ``error`` value at the center of
-            the aperture is used for the entire aperture.
-
         mask : array_like (bool), optional
             A boolean mask with the same shape as ``data`` where a
             `True` value indicates the corresponding element of ``data``
@@ -418,20 +407,7 @@ class PixelAperture(Aperture):
                 if error_cutout is None:
                     aperture_sum_errs.append(np.nan)
                 else:
-                    if pixelwise_error:
-                        aperture_var = np.sum(error_cutout ** 2 * mask.data)
-                    else:
-                        # use central value (shifted for partial overlap)
-                        _, slc_sm = mask._overlap_slices(error.shape)
-                        yidx = int((slc_sm[0].start + slc_sm[0].stop - 1) /
-                                   2. + 0.5)
-                        xidx = int((slc_sm[1].start + slc_sm[1].stop - 1) /
-                                   2. + 0.5)
-                        error_value = error_cutout[yidx, xidx]
-
-                        aperture_var = np.sum(error_value ** 2 *
-                                              np.sum(mask.data))
-
+                    aperture_var = np.sum(error_cutout ** 2 * mask.data)
                     aperture_sum_errs.append(np.sqrt(aperture_var))
 
         # handle Quantity objects and input units
@@ -442,6 +418,33 @@ class PixelAperture(Aperture):
 
         return aperture_sums, aperture_sum_errs
 
+    @staticmethod
+    def _make_annulus_path(patch_inner, patch_outer):
+        """
+        Define a matplotlib annulus path from two patches.
+
+        This preserves the cubic Bezier curves (CURVE4) of the aperture
+        paths.
+        """
+
+        import matplotlib.path as mpath
+
+        path_inner = patch_inner.get_path()
+        transform_inner = patch_inner.get_transform()
+        path_inner = transform_inner.transform_path(path_inner)
+
+        path_outer = patch_outer.get_path()
+        transform_outer = patch_outer.get_transform()
+        path_outer = transform_outer.transform_path(path_outer)
+
+        verts_inner = path_inner.vertices[:-1][::-1]
+        verts_inner = np.concatenate((verts_inner, [verts_inner[-1]]))
+
+        verts = np.vstack((path_outer.vertices, verts_inner))
+        codes = np.hstack((path_outer.codes, path_inner.codes))
+
+        return mpath.Path(verts, codes)
+
     def _prepare_plot(self, origin=(0, 0), indices=None, ax=None,
                       fill=False, **kwargs):
         """
@@ -527,201 +530,160 @@ class PixelAperture(Aperture):
             Any keyword arguments accepted by `matplotlib.patches.Patch`.
         """
 
+        raise NotImplementedError('Needs to be implemented in a '
+                                  'PixelAperture subclass.')
 
-class ApertureMask(object):
+    def _to_sky_params(self, wcs, mode='all'):
+        """
+        Convert the pixel aperture parameters to those for a sky
+        aperture.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        sky_params : dict
+            A dictionary of parameters for an equivalent sky aperture.
+        """
+
+        sky_params = {}
+        x, y = np.transpose(self.positions)
+        sky_params['positions'] = pixel_to_skycoord(x, y, wcs, mode=mode)
+
+        # The aperture object must have a single value for each shape
+        # parameter so we must use a single pixel scale for all positions.
+        # Here, we define the scale at the WCS CRVAL position.
+        crval = SkyCoord([wcs.wcs.crval], frame=wcs_to_celestial_frame(wcs),
+                         unit=wcs.wcs.cunit)
+        scale, angle = pixel_scale_angle_at_skycoord(crval, wcs)
+
+        params = self._params[:]
+        theta_key = 'theta'
+        if theta_key in self._params:
+            sky_params[theta_key] = (self.theta * u.rad) - angle.to(u.rad)
+            params.remove(theta_key)
+
+        param_vals = [getattr(self, param) for param in params]
+        for param, param_val in zip(params, param_vals):
+            sky_params[param] = (param_val * u.pix * scale).to(u.arcsec)
+
+        return sky_params
+
+    @abc.abstractmethod
+    def to_sky(self, wcs, mode='all'):
+        """
+        Convert the aperture to a `SkyAperture` object defined in
+        celestial coordinates.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS` object
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        aperture : `SkyAperture` object
+            A `SkyAperture` object.
+        """
+
+        raise NotImplementedError('Needs to be implemented in a '
+                                  'PixelAperture subclass.')
+
+
+class SkyAperture(Aperture):
     """
-    Class for an aperture mask.
-
-    Parameters
-    ----------
-    mask : array_like
-        A 2D array of an aperture mask representing the fractional
-        overlap of the aperture on the pixel grid.  This should be the
-        full-sized (i.e. not truncated) array that is the direct output
-        of one of the low-level `photutils.geometry` functions.
-
-    bbox_slice : tuple of slice objects
-        A tuple of ``(y, x)`` numpy slice objects defining the aperture
-        minimal bounding box.
+    Abstract base class for all apertures defined in celestial
+    coordinates.
     """
 
-    def __init__(self, mask, bbox_slice):
-        self.data = np.asanyarray(mask)
-        self.shape = mask.shape
-        self.slices = bbox_slice
-
-        dy = bbox_slice[0].stop - bbox_slice[0].start
-        dx = bbox_slice[1].stop - bbox_slice[1].start
-        if self.data.shape != (dy, dx):
-            raise ValueError('mask shape and bbox_slice do not agree.')
-
-    @property
-    def array(self):
-        """The 2D mask array."""
-
-        return self.data
-
-    def __array__(self):
+    def _to_pixel_params(self, wcs, mode='all'):
         """
-        Array representation of the mask array (e.g., for matplotlib).
-        """
-
-        return self.data
-
-    def _overlap_slices(self, shape):
-        """
-        Calculate the slices for the overlapping part of ``self.slices``
-        and an array of the given shape.
+        Convert the sky aperture parameters to those for a pixel
+        aperture.
 
         Parameters
         ----------
-        shape : tuple of int
-            The ``(ny, nx)`` shape of array where the slices are to be
-            applied.
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
 
         Returns
         -------
-        slices_large : tuple of slices
-            A tuple of slice objects for each axis of the large array,
-            such that ``large_array[slices_large]`` extracts the region
-            of the large array that overlaps with the small array.
-
-        slices_small : slice
-            A tuple of slice objects for each axis of the small array,
-            such that ``small_array[slices_small]`` extracts the region
-            of the small array that is inside the large array.
+        pixel_params : dict
+            A dictionary of parameters for an equivalent pixel aperture.
         """
 
-        if len(shape) != 2:
-            raise ValueError('input shape must have 2 elements.')
+        pixel_params = {}
+        x, y = skycoord_to_pixel(self.positions, wcs, mode=mode)
+        pixel_params['positions'] = np.array([x, y]).transpose()
 
-        ymin = self.slices[0].start
-        ymax = self.slices[0].stop
-        xmin = self.slices[1].start
-        xmax = self.slices[1].stop
+        # The aperture object must have a single value for each shape
+        # parameter so we must use a single pixel scale for all positions.
+        # Here, we define the scale at the WCS CRVAL position.
+        crval = SkyCoord([wcs.wcs.crval], frame=wcs_to_celestial_frame(wcs),
+                         unit=wcs.wcs.cunit)
+        scale, angle = pixel_scale_angle_at_skycoord(crval, wcs)
 
-        if (xmin >= shape[1] or ymin >= shape[0] or xmax <= 0 or ymax <= 0):
-            # no overlap of the aperture with the data
-            return None, None
+        params = self._params[:]
+        theta_key = 'theta'
+        if theta_key in self._params:
+            pixel_params[theta_key] = (self.theta + angle).to(u.radian).value
+            params.remove(theta_key)
 
-        slices_large = (slice(max(ymin, 0), min(ymax, shape[0])),
-                        slice(max(xmin, 0), min(xmax, shape[1])))
+        param_vals = [getattr(self, param) for param in params]
+        if param_vals[0].unit.physical_type == 'angle':
+            for param, param_val in zip(params, param_vals):
+                pixel_params[param] = (param_val / scale).to(u.pixel).value
+        else:    # pixels
+            for param, param_val in zip(params, param_vals):
+                pixel_params[param] = param_val.value
 
-        slices_small = (slice(max(-ymin, 0),
-                              min(ymax - ymin, shape[0] - ymin)),
-                        slice(max(-xmin, 0),
-                              min(xmax - xmin, shape[1] - xmin)))
+        return pixel_params
 
-        return slices_large, slices_small
-
-    def to_image(self, shape):
+    @abc.abstractmethod
+    def to_pixel(self, wcs, mode='all'):
         """
-        Return an image of the mask in a 2D array of the given shape,
-        taking any edge effects into account.
+        Convert the aperture to a `PixelAperture` object defined in
+        pixel coordinates.
 
         Parameters
         ----------
-        shape : tuple of int
-            The ``(ny, nx)`` shape of the output array.
+        wcs : `~astropy.wcs.WCS` object
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or including only the core WCS
+            transformation (``'wcs'``).
 
         Returns
         -------
-        result : `~numpy.ndarray`
-            A 2D array of the mask.
+        aperture : `PixelAperture` object
+            A `PixelAperture` object.
         """
 
-        if len(shape) != 2:
-            raise ValueError('input shape must have 2 elements.')
-
-        mask = np.zeros(shape)
-
-        try:
-            mask[self.slices] = self.data
-        except ValueError:    # partial or no overlap
-            slices_large, slices_small = self._overlap_slices(shape)
-
-            if slices_small is None:
-                return None    # no overlap
-
-            mask = np.zeros(shape)
-            mask[slices_large] = self.data[slices_small]
-
-        return mask
-
-    def cutout(self, data, fill_value=0.):
-        """
-        Create a cutout from the input data over the mask bounding box,
-        taking any edge effects into account.
-
-        Parameters
-        ----------
-        data : array_like or `~astropy.units.Quantity`
-            A 2D array on which to apply the aperture mask.
-
-        fill_value : float, optional
-            The value is used to fill pixels where the aperture mask
-            does not overlap with the input ``data``.  The default is 0.
-
-        Returns
-        -------
-        result : `~numpy.ndarray`
-            A 2D array cut out from the input ``data`` representing the
-            same cutout region as the aperture mask.  If there is a
-            partial overlap of the aperture mask with the input data,
-            pixels outside of the data will be assigned to
-            ``fill_value``.  `None` is returned if there is no overlap
-            of the aperture with the input ``data``.
-        """
-
-        data = np.asanyarray(data)
-        cutout = data[self.slices]
-
-        if cutout.shape != self.shape:
-            slices_large, slices_small = self._overlap_slices(data.shape)
-
-            if slices_small is None:
-                return None    # no overlap
-
-            cutout = np.zeros(self.shape, dtype=data.dtype)
-            cutout[:] = fill_value
-            cutout[slices_small] = data[slices_large]
-
-            if isinstance(data, u.Quantity):
-                cutout = u.Quantity(cutout, unit=data.unit)
-
-        return cutout
-
-    def apply(self, data, fill_value=0.):
-        """
-        Apply the aperture mask to the input data, taking any edge
-        effects into account.
-
-        The result is a mask-weighted cutout from the data.
-
-        Parameters
-        ----------
-        data : array_like or `~astropy.units.Quantity`
-            A 2D array on which to apply the aperture mask.
-
-        fill_value : float, optional
-            The value is used to fill pixels where the aperture mask
-            does not overlap with the input ``data``.  The default is 0.
-
-        Returns
-        -------
-        result : `~numpy.ndarray`
-            A 2D mask-weighted cutout from the input ``data``.  If there
-            is a partial overlap of the aperture mask with the input
-            data, pixels outside of the data will be assigned to
-            ``fill_value`` before being multipled with the mask.  `None`
-            is returned if there is no overlap of the aperture with the
-            input ``data``.
-        """
-
-        return self.cutout(data, fill_value=fill_value) * self.data
+        raise NotImplementedError('Needs to be implemented in a '
+                                  'SkyAperture subclass.')
 
 
-def _prepare_photometry_input(data, error, pixelwise_error, mask, wcs, unit):
+def _prepare_photometry_input(data, error, mask, wcs, unit):
     """
     Parse the inputs to `aperture_photometry`.
 
@@ -758,7 +720,7 @@ def _prepare_photometry_input(data, error, pixelwise_error, mask, wcs, unit):
     if wcs is None:
         try:
             wcs = WCS(header)
-        except:
+        except Exception:
             # A valid WCS was not found in the header.  Let the calling
             # application raise an exception if it needs a WCS.
             pass
@@ -791,17 +753,9 @@ def _prepare_photometry_input(data, error, pixelwise_error, mask, wcs, unit):
             if np.isscalar(error.value):
                 error = u.Quantity(np.broadcast_arrays(error, data),
                                    unit=error.unit)[0]
-                pixelwise_error = False
-                warnings.warn('Because input error was a scalar, '
-                              'pixelwise_error=False is being used.',
-                              AstropyUserWarning)
         else:
             if np.isscalar(error):
                 error = np.broadcast_arrays(error, data)[0]
-                pixelwise_error = False
-                warnings.warn('Because input error was a scalar, '
-                              'pixelwise_error=False is being used.',
-                              AstropyUserWarning)
 
             if unit is not None:
                 error = u.Quantity(error, unit=unit)
@@ -816,13 +770,12 @@ def _prepare_photometry_input(data, error, pixelwise_error, mask, wcs, unit):
         if mask.shape != data.shape:
             raise ValueError('mask and data must have the same shape.')
 
-    return data, error, pixelwise_error, mask, wcs
+    return data, error, mask, wcs
 
 
 @support_nddata
-def aperture_photometry(data, apertures, error=None, pixelwise_error=True,
-                        mask=None, method='exact', subpixels=5, unit=None,
-                        wcs=None):
+def aperture_photometry(data, apertures, error=None, mask=None,
+                        method='exact', subpixels=5, unit=None, wcs=None):
     """
     Perform aperture photometry on the input data by summing the flux
     within the given aperture(s).
@@ -847,12 +800,6 @@ def aperture_photometry(data, apertures, error=None, pixelwise_error=True,
         `~photutils.utils.calc_total_error`) .  ``error`` must have the
         same shape as the input ``data``.
 
-    pixelwise_error : bool, optional
-        If `True` (default), the photometric error is calculated using
-        the ``error`` values from each pixel within the aperture.  If
-        `False`, the ``error`` value at the center of the aperture is
-        used for the entire aperture.
-
     mask : array_like (bool), optional
         A boolean mask with the same shape as ``data`` where a `True`
         value indicates the corresponding element of ``data`` is masked.
@@ -938,9 +885,8 @@ def aperture_photometry(data, apertures, error=None, pixelwise_error=True,
     thus supports `~astropy.nddata.NDData` objects as input.
     """
 
-    data, error, pixelwise_error, mask, wcs = \
-        _prepare_photometry_input(data, error, pixelwise_error, mask, wcs,
-                                  unit)
+    data, error, mask, wcs = _prepare_photometry_input(data, error, mask,
+                                                       wcs, unit)
 
     if method == 'subpixel':
         if (int(subpixels) != subpixels) or (subpixels <= 0):
@@ -969,11 +915,10 @@ def aperture_photometry(data, apertures, error=None, pixelwise_error=True,
             raise ValueError('Input apertures must all have identical '
                              'positions.')
 
-    calling_args = ("method='{0}', subpixels={1}, pixelwise_error={2}"
-                    .format(method, subpixels, pixelwise_error))
     meta = OrderedDict()
     meta['name'] = 'Aperture photometry results'
-    meta['version'] = _get_version_info()
+    meta['version'] = get_version_info()
+    calling_args = ("method='{0}', subpixels={1}".format(method, subpixels))
     meta['aperture_photometry_args'] = calling_args
 
     tbl = QTable(meta=meta)
@@ -990,9 +935,9 @@ def aperture_photometry(data, apertures, error=None, pixelwise_error=True,
             tbl['celestial_center'] = skycoord_pos
 
     for i, aper in enumerate(apertures):
-        aper_sum, aper_sum_err = aper.do_photometry(
-            data, error=error, pixelwise_error=pixelwise_error, mask=mask,
-            method=method, subpixels=subpixels)
+        aper_sum, aper_sum_err = aper.do_photometry(data, error=error,
+                                                    mask=mask, method=method,
+                                                    subpixels=subpixels)
 
         sum_key = 'aperture_sum'
         sum_err_key = 'aperture_sum_err'
