@@ -5,15 +5,12 @@ import math
 
 import numpy as np
 from astropy.coordinates import SkyCoord
-import astropy.units as u
-from astropy.wcs.utils import skycoord_to_pixel
 
-from .core import (PixelAperture, SkyAperture, ApertureMask,
-                   _sanitize_pixel_positions, _translate_mask_method,
-                   _make_annulus_path)
+from .core import PixelAperture, SkyAperture
+from .bounding_box import BoundingBox
+from .mask import ApertureMask
 from ..geometry import elliptical_overlap_grid
-from ..utils.wcs_helpers import (skycoord_to_pixel_scale_angle, assert_angle,
-                                 assert_angle_or_pixel)
+from ..utils.wcs_helpers import assert_angle, assert_angle_or_pixel
 
 
 __all__ = ['EllipticalMaskMixin', 'EllipticalAperture', 'EllipticalAnnulus',
@@ -28,8 +25,8 @@ class EllipticalMaskMixin(object):
 
     def to_mask(self, method='exact', subpixels=5):
         """
-        Return a list of `ApertureMask` objects, one for each aperture
-        position.
+        Return a list of `~photutils.ApertureMask` objects, one for each
+        aperture position.
 
         Parameters
         ----------
@@ -70,11 +67,7 @@ class EllipticalMaskMixin(object):
             A list of aperture mask objects.
         """
 
-        if method not in ('center', 'subpixel', 'exact'):
-            raise ValueError('"{0}" method is not available for this '
-                             'aperture.'.format(method))
-
-        use_exact, subpixels = _translate_mask_method(method, subpixels)
+        use_exact, subpixels = self._translate_mask_mode(method, subpixels)
 
         if hasattr(self, 'a'):
             a = self.a
@@ -87,24 +80,20 @@ class EllipticalMaskMixin(object):
             raise ValueError('Cannot determine the aperture shape.')
 
         masks = []
-        for position, _slice, _geom_slice in zip(self.positions, self._slices,
-                                                 self._geom_slices):
-            px_min, px_max = _geom_slice[1].start, _geom_slice[1].stop
-            py_min, py_max = _geom_slice[0].start, _geom_slice[0].stop
-            dx = px_max - px_min
-            dy = py_max - py_min
-
-            mask = elliptical_overlap_grid(px_min, px_max, py_min, py_max,
-                                           dx, dy, a, b, self.theta,
+        for bbox, edges in zip(self.bounding_boxes, self._centered_edges):
+            ny, nx = bbox.shape
+            mask = elliptical_overlap_grid(edges[0], edges[1], edges[2],
+                                           edges[3], nx, ny, a, b, self.theta,
                                            use_exact, subpixels)
 
-            if hasattr(self, 'a_in'):    # annulus
-                mask -= elliptical_overlap_grid(px_min, px_max, py_min,
-                                                py_max, dx, dy, self.a_in,
+            # subtract the inner ellipse for an annulus
+            if hasattr(self, 'a_in'):
+                mask -= elliptical_overlap_grid(edges[0], edges[1], edges[2],
+                                                edges[3], nx, ny, self.a_in,
                                                 b_in, self.theta, use_exact,
                                                 subpixels)
 
-            masks.append(ApertureMask(mask, _slice))
+            masks.append(ApertureMask(mask, bbox))
 
         return masks
 
@@ -146,31 +135,38 @@ class EllipticalAperture(EllipticalMaskMixin, PixelAperture):
     """
 
     def __init__(self, positions, a, b, theta):
-        try:
-            self.a = float(a)
-            self.b = float(b)
-            self.theta = float(theta)
-        except TypeError:
-            raise TypeError("'a' and 'b' and 'theta' must be numeric, "
-                            "received {0} and {1} and {2}."
-                            .format((type(a), type(b), type(theta))))
-
         if a < 0 or b < 0:
             raise ValueError("'a' and 'b' must be non-negative.")
 
-        self.positions = _sanitize_pixel_positions(positions)
+        self.positions = self._sanitize_positions(positions)
+        self.a = float(a)
+        self.b = float(b)
+        self.theta = float(theta)
+        self._params = ['a', 'b', 'theta']
 
     @property
-    def _slices(self):
-        # TODO:  use an actual minimal bounding box
-        radius = max(self.a, self.b)
-        x_min = np.floor(self.positions[:, 0] - radius + 0.5).astype(int)
-        x_max = np.floor(self.positions[:, 0] + radius + 1.5).astype(int)
-        y_min = np.floor(self.positions[:, 1] - radius + 0.5).astype(int)
-        y_max = np.floor(self.positions[:, 1] + radius + 1.5).astype(int)
+    def bounding_boxes(self):
+        """
+        A list of minimal bounding boxes (`~photutils.BoundingBox`), one
+        for each position, enclosing the exact elliptical apertures.
+        """
 
-        return [(slice(ymin, ymax), slice(xmin, xmax))
-                for xmin, xmax, ymin, ymax in zip(x_min, x_max, y_min, y_max)]
+        cos_theta = np.cos(self.theta)
+        sin_theta = np.sin(self.theta)
+        ax = self.a * cos_theta
+        ay = self.a * sin_theta
+        bx = self.b * -sin_theta
+        by = self.b * cos_theta
+        dx = np.sqrt(ax*ax + bx*bx)
+        dy = np.sqrt(ay*ay + by*by)
+
+        xmin = self.positions[:, 0] - dx
+        xmax = self.positions[:, 0] + dx
+        ymin = self.positions[:, 1] - dy
+        ymax = self.positions[:, 1] + dy
+
+        return [BoundingBox._from_float(x0, x1, y0, y1)
+                for x0, x1, y0, y1 in zip(xmin, xmax, ymin, ymax)]
 
     def area(self):
         return math.pi * self.a * self.b
@@ -188,6 +184,30 @@ class EllipticalAperture(EllipticalMaskMixin, PixelAperture):
                                      theta_deg, **kwargs)
             ax.add_patch(patch)
 
+    def to_sky(self, wcs, mode='all'):
+        """
+        Convert the aperture to a `SkyEllipticalAperture` object defined
+        in celestial coordinates.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        aperture : `SkyEllipticalAperture` object
+            A `SkyEllipticalAperture` object.
+        """
+
+        sky_params = self._to_sky_params(wcs, mode=mode)
+        return SkyEllipticalAperture(**sky_params)
+
 
 class EllipticalAnnulus(EllipticalMaskMixin, PixelAperture):
     """
@@ -238,37 +258,42 @@ class EllipticalAnnulus(EllipticalMaskMixin, PixelAperture):
     """
 
     def __init__(self, positions, a_in, a_out, b_out, theta):
-        try:
-            self.a_in = float(a_in)
-            self.a_out = float(a_out)
-            self.b_out = float(b_out)
-            self.theta = float(theta)
-        except TypeError:
-            raise TypeError("'a_in' and 'a_out' and 'b_out' and 'theta' must "
-                            "be numeric, received {0} and {1} and {2} and "
-                            "{3}.".format((type(a_in), type(a_out),
-                                           type(b_out), type(theta))))
-
         if not (a_out > a_in):
-            raise ValueError("'a_out' must be greater than 'a_in'")
+            raise ValueError('"a_out" must be greater than "a_in".')
         if a_in < 0 or b_out < 0:
-            raise ValueError("'a_in' and 'b_out' must be non-negative")
+            raise ValueError('"a_in" and "b_out" must be non-negative.')
 
-        self.b_in = b_out * a_in / a_out
-
-        self.positions = _sanitize_pixel_positions(positions)
+        self.positions = self._sanitize_positions(positions)
+        self.a_in = float(a_in)
+        self.a_out = float(a_out)
+        self.b_out = float(b_out)
+        self.b_in = self.b_out * self.a_in / self.a_out
+        self.theta = float(theta)
+        self._params = ['a_in', 'a_out', 'b_out', 'theta']
 
     @property
-    def _slices(self):
-        # TODO:  use an actual minimal bounding box
-        radius = max(self.a_out, self.b_out)
-        x_min = np.floor(self.positions[:, 0] - radius + 0.5).astype(int)
-        x_max = np.floor(self.positions[:, 0] + radius + 1.5).astype(int)
-        y_min = np.floor(self.positions[:, 1] - radius + 0.5).astype(int)
-        y_max = np.floor(self.positions[:, 1] + radius + 1.5).astype(int)
+    def bounding_boxes(self):
+        """
+        A list of minimal bounding boxes (`~photutils.BoundingBox`), one
+        for each position, enclosing the exact elliptical apertures.
+        """
 
-        return [(slice(ymin, ymax), slice(xmin, xmax))
-                for xmin, xmax, ymin, ymax in zip(x_min, x_max, y_min, y_max)]
+        cos_theta = np.cos(self.theta)
+        sin_theta = np.sin(self.theta)
+        ax = self.a_out * cos_theta
+        ay = self.a_out * sin_theta
+        bx = self.b_out * -sin_theta
+        by = self.b_out * cos_theta
+        dx = np.sqrt(ax*ax + bx*bx)
+        dy = np.sqrt(ay*ay + by*by)
+
+        xmin = self.positions[:, 0] - dx
+        xmax = self.positions[:, 0] + dx
+        ymin = self.positions[:, 1] - dy
+        ymax = self.positions[:, 1] + dy
+
+        return [BoundingBox._from_float(x0, x1, y0, y1)
+                for x0, x1, y0, y1 in zip(xmin, xmax, ymin, ymax)]
 
     def area(self):
         return math.pi * (self.a_out * self.b_out - self.a_in * self.b_in)
@@ -286,10 +311,34 @@ class EllipticalAnnulus(EllipticalMaskMixin, PixelAperture):
                                            2.*self.b_in, theta_deg, **kwargs)
             patch_outer = mpatches.Ellipse(position, 2.*self.a_out,
                                            2.*self.b_out, theta_deg, **kwargs)
-            path = _make_annulus_path(patch_inner, patch_outer)
+            path = self._make_annulus_path(patch_inner, patch_outer)
             patch = mpatches.PathPatch(path, **kwargs)
             ax.add_patch(patch)
 
+    def to_sky(self, wcs, mode='all'):
+        """
+        Convert the aperture to a `SkyEllipticalAnnulus` object defined
+        in celestial coordinates.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        aperture : `SkyEllipticalAnnulus` object
+            A `SkyEllipticalAnnulus` object.
+        """
+
+        sky_params = self._to_sky_params(wcs, mode=mode)
+        return SkyEllipticalAnnulus(**sky_params)
+
 
 class SkyEllipticalAperture(SkyAperture):
     """
@@ -317,7 +366,7 @@ class SkyEllipticalAperture(SkyAperture):
         if isinstance(positions, SkyCoord):
             self.positions = positions
         else:
-            raise TypeError("positions should be a SkyCoord instance")
+            raise TypeError('positions must be a SkyCoord instance')
 
         assert_angle_or_pixel('a', a)
         assert_angle_or_pixel('b', b)
@@ -330,16 +379,17 @@ class SkyEllipticalAperture(SkyAperture):
         self.a = a
         self.b = b
         self.theta = theta
+        self._params = ['a', 'b', 'theta']
 
     def to_pixel(self, wcs, mode='all'):
         """
-        Convert the aperture to an `EllipticalAperture` instance in
-        pixel coordinates.
+        Convert the aperture to an `EllipticalAperture` object defined
+        in pixel coordinates.
 
         Parameters
         ----------
         wcs : `~astropy.wcs.WCS`
-            The WCS transformation to use.
+            The world coordinate system (WCS) transformation to use.
 
         mode : {'all', 'wcs'}, optional
             Whether to do the transformation including distortions
@@ -352,23 +402,8 @@ class SkyEllipticalAperture(SkyAperture):
             An `EllipticalAperture` object.
         """
 
-        x, y = skycoord_to_pixel(self.positions, wcs, mode=mode)
-        central_pos = SkyCoord([wcs.wcs.crval], frame=self.positions.name,
-                               unit=wcs.wcs.cunit)
-        xc, yc, scale, angle = skycoord_to_pixel_scale_angle(central_pos, wcs)
-
-        if self.a.unit.physical_type == 'angle':
-            a = (scale * self.a).to(u.pixel).value
-            b = (scale * self.b).to(u.pixel).value
-        else:  # pixel
-            a = self.a.value
-            b = self.b.value
-
-        theta = (angle + self.theta).to(u.radian).value
-
-        pixel_positions = np.array([x, y]).transpose()
-
-        return EllipticalAperture(pixel_positions, a, b, theta)
+        pixel_params = self._to_pixel_params(wcs, mode=mode)
+        return EllipticalAperture(**pixel_params)
 
 
 class SkyEllipticalAnnulus(SkyAperture):
@@ -404,7 +439,7 @@ class SkyEllipticalAnnulus(SkyAperture):
         if isinstance(positions, SkyCoord):
             self.positions = positions
         else:
-            raise TypeError("positions should be a SkyCoord instance")
+            raise TypeError('positions must be a SkyCoord instance')
 
         assert_angle_or_pixel('a_in', a_in)
         assert_angle_or_pixel('a_out', a_out)
@@ -423,16 +458,17 @@ class SkyEllipticalAnnulus(SkyAperture):
         self.a_out = a_out
         self.b_out = b_out
         self.theta = theta
+        self._params = ['a_in', 'a_out', 'b_out', 'theta']
 
     def to_pixel(self, wcs, mode='all'):
         """
-        Convert the aperture to an `EllipticalAnnulus` instance in pixel
-        coordinates.
+        Convert the aperture to an `EllipticalAnnulus` object defined in
+        pixel coordinates.
 
         Parameters
         ----------
         wcs : `~astropy.wcs.WCS`
-            The WCS transformation to use.
+            The world coordinate system (WCS) transformation to use.
 
         mode : {'all', 'wcs'}, optional
             Whether to do the transformation including distortions
@@ -445,22 +481,5 @@ class SkyEllipticalAnnulus(SkyAperture):
             An `EllipticalAnnulus` object.
         """
 
-        x, y = skycoord_to_pixel(self.positions, wcs, mode=mode)
-        central_pos = SkyCoord([wcs.wcs.crval], frame=self.positions.name,
-                               unit=wcs.wcs.cunit)
-        xc, yc, scale, angle = skycoord_to_pixel_scale_angle(central_pos, wcs)
-
-        if self.a_in.unit.physical_type == 'angle':
-            a_in = (scale * self.a_in).to(u.pixel).value
-            a_out = (scale * self.a_out).to(u.pixel).value
-            b_out = (scale * self.b_out).to(u.pixel).value
-        else:
-            a_in = self.a_in.value
-            a_out = self.a_out.value
-            b_out = self.b_out.value
-
-        theta = (angle + self.theta).to(u.radian).value
-
-        pixel_positions = np.array([x, y]).transpose()
-
-        return EllipticalAnnulus(pixel_positions, a_in, a_out, b_out, theta)
+        pixel_params = self._to_pixel_params(wcs, mode=mode)
+        return EllipticalAnnulus(**pixel_params)
