@@ -2,20 +2,15 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import math
-import warnings
 
 import numpy as np
 from astropy.coordinates import SkyCoord
-import astropy.units as u
-from astropy.wcs.utils import skycoord_to_pixel
-from astropy.utils.exceptions import AstropyUserWarning
 
-from .core import (PixelAperture, SkyAperture, ApertureMask,
-                   _sanitize_pixel_positions, _translate_mask_method,
-                   _make_annulus_path)
+from .core import PixelAperture, SkyAperture
+from .bounding_box import BoundingBox
+from .mask import ApertureMask
 from ..geometry import rectangular_overlap_grid
-from ..utils.wcs_helpers import (skycoord_to_pixel_scale_angle, assert_angle,
-                                 assert_angle_or_pixel)
+from ..utils.wcs_helpers import assert_angle, assert_angle_or_pixel
 
 
 __all__ = ['RectangularMaskMixin', 'RectangularAperture',
@@ -31,8 +26,8 @@ class RectangularMaskMixin(object):
 
     def to_mask(self, method='exact', subpixels=5):
         """
-        Return a list of `ApertureMask` objects, one for each aperture
-        position.
+        Return a list of `~photutils.ApertureMask` objects, one for each
+        aperture position.
 
         Parameters
         ----------
@@ -73,17 +68,8 @@ class RectangularMaskMixin(object):
             A list of aperture mask objects.
         """
 
-        if method == 'exact':
-            warnings.warn("'exact' method is not implemented for rectangular "
-                          "apertures -- instead using 'subpixel' method with "
-                          "subpixels=32", AstropyUserWarning)
-            method = 'subpixel'
-            subpixels = 32
-        elif method not in ('center', 'subpixel'):
-            raise ValueError('"{0}" method is not available for this '
-                             'aperture.'.format(method))
-
-        use_exact, subpixels = _translate_mask_method(method, subpixels)
+        use_exact, subpixels = self._translate_mask_mode(method, subpixels,
+                                                         rectangle=True)
 
         if hasattr(self, 'w'):
             w = self.w
@@ -96,24 +82,20 @@ class RectangularMaskMixin(object):
             raise ValueError('Cannot determine the aperture radius.')
 
         masks = []
-        for position, _slice, _geom_slice in zip(self.positions, self._slices,
-                                                 self._geom_slices):
-            px_min, px_max = _geom_slice[1].start, _geom_slice[1].stop
-            py_min, py_max = _geom_slice[0].start, _geom_slice[0].stop
-            dx = px_max - px_min
-            dy = py_max - py_min
+        for bbox, edges in zip(self.bounding_boxes, self._centered_edges):
+            ny, nx = bbox.shape
+            mask = rectangular_overlap_grid(edges[0], edges[1], edges[2],
+                                            edges[3], nx, ny, w, h,
+                                            self.theta, 0, subpixels)
 
-            mask = rectangular_overlap_grid(px_min, px_max, py_min, py_max,
-                                            dx, dy, w, h, self.theta, 0,
-                                            subpixels)
+            # subtract the inner circle for an annulus
+            if hasattr(self, 'w_in'):
+                mask -= rectangular_overlap_grid(edges[0], edges[1], edges[2],
+                                                 edges[3], nx, ny, self.w_in,
+                                                 h_in, self.theta, 0,
+                                                 subpixels)
 
-            if hasattr(self, 'w_in'):    # annulus
-                mask -= rectangular_overlap_grid(px_min, px_max, py_min,
-                                                 py_max, dx, dy,
-                                                 self.w_in, h_in, self.theta,
-                                                 0, subpixels)
-
-            masks.append(ApertureMask(mask, _slice))
+            masks.append(ApertureMask(mask, bbox))
 
         return masks
 
@@ -157,30 +139,40 @@ class RectangularAperture(RectangularMaskMixin, PixelAperture):
     """
 
     def __init__(self, positions, w, h, theta):
-        try:
-            self.w = float(w)
-            self.h = float(h)
-            self.theta = float(theta)
-        except TypeError:
-            raise TypeError("'w' and 'h' and 'theta' must "
-                            "be numeric, received {0} and {1} and {2}."
-                            .format((type(w), type(h), type(theta))))
         if w < 0 or h < 0:
             raise ValueError("'w' and 'h' must be nonnegative.")
 
-        self.positions = _sanitize_pixel_positions(positions)
+        self.positions = self._sanitize_positions(positions)
+        self.w = float(w)
+        self.h = float(h)
+        self.theta = float(theta)
+        self._params = ['w', 'h', 'theta']
 
     @property
-    def _slices(self):
-        # TODO:  use an actual minimal bounding box
-        radius = max(self.h, self.w) * (2. ** -0.5)
-        x_min = np.floor(self.positions[:, 0] - radius + 0.5).astype(int)
-        x_max = np.floor(self.positions[:, 0] + radius + 1.5).astype(int)
-        y_min = np.floor(self.positions[:, 1] - radius + 0.5).astype(int)
-        y_max = np.floor(self.positions[:, 1] + radius + 1.5).astype(int)
+    def bounding_boxes(self):
+        """
+        A list of minimal bounding boxes (`~photutils.BoundingBox`), one
+        for each position, enclosing the exact rectangular apertures.
+        """
 
-        return [(slice(ymin, ymax), slice(xmin, xmax))
-                for xmin, xmax, ymin, ymax in zip(x_min, x_max, y_min, y_max)]
+        w2 = self.w / 2.
+        h2 = self.h / 2.
+        cos_theta = math.cos(self.theta)
+        sin_theta = math.sin(self.theta)
+        dx1 = abs(w2 * cos_theta - h2 * sin_theta)
+        dy1 = abs(w2 * sin_theta + h2 * cos_theta)
+        dx2 = abs(w2 * cos_theta + h2 * sin_theta)
+        dy2 = abs(w2 * sin_theta - h2 * cos_theta)
+        dx = max(dx1, dx2)
+        dy = max(dy1, dy2)
+
+        xmin = self.positions[:, 0] - dx
+        xmax = self.positions[:, 0] + dx
+        ymin = self.positions[:, 1] - dy
+        ymax = self.positions[:, 1] + dy
+
+        return [BoundingBox._from_float(x0, x1, y0, y1)
+                for x0, x1, y0, y1 in zip(xmin, xmax, ymin, ymax)]
 
     def area(self):
         return self.w * self.h
@@ -205,6 +197,30 @@ class RectangularAperture(RectangularMaskMixin, PixelAperture):
                                        **kwargs)
             ax.add_patch(patch)
 
+    def to_sky(self, wcs, mode='all'):
+        """
+        Convert the aperture to a `SkyRectangularAperture` object
+        defined in celestial coordinates.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        aperture : `SkyRectangularAperture` object
+            A `SkyRectangularAperture` object.
+        """
+
+        sky_params = self._to_sky_params(wcs, mode=mode)
+        return SkyRectangularAperture(**sky_params)
+
 
 class RectangularAnnulus(RectangularMaskMixin, PixelAperture):
     """
@@ -259,37 +275,45 @@ class RectangularAnnulus(RectangularMaskMixin, PixelAperture):
     """
 
     def __init__(self, positions, w_in, w_out, h_out, theta):
-        try:
-            self.w_in = float(w_in)
-            self.w_out = float(w_out)
-            self.h_out = float(h_out)
-            self.theta = float(theta)
-        except TypeError:
-            raise TypeError("'w_in' and 'w_out' and 'h_out' and 'theta' must "
-                            "be numeric, received {0} and {1} and {2} and "
-                            "{3}.".format((type(w_in), type(w_out),
-                                           type(h_out), type(theta))))
-
         if not (w_out > w_in):
             raise ValueError("'w_out' must be greater than 'w_in'")
         if w_in < 0 or h_out < 0:
             raise ValueError("'w_in' and 'h_out' must be non-negative")
 
-        self.h_in = w_in * h_out / w_out
-
-        self.positions = _sanitize_pixel_positions(positions)
+        self.positions = self._sanitize_positions(positions)
+        self.w_in = float(w_in)
+        self.w_out = float(w_out)
+        self.h_out = float(h_out)
+        self.h_in = self.w_in * self.h_out / self.w_out
+        self.theta = float(theta)
+        self._params = ['w_in', 'w_out', 'h_out', 'theta']
 
     @property
-    def _slices(self):
-        # TODO:  use an actual minimal bounding box
-        radius = max(self.h_out, self.w_out) * (2. ** -0.5)
-        x_min = np.floor(self.positions[:, 0] - radius + 0.5).astype(int)
-        x_max = np.floor(self.positions[:, 0] + radius + 1.5).astype(int)
-        y_min = np.floor(self.positions[:, 1] - radius + 0.5).astype(int)
-        y_max = np.floor(self.positions[:, 1] + radius + 1.5).astype(int)
+    def bounding_boxes(self):
+        """
+        A list of minimal bounding boxes (`~photutils.BoundingBox`), one
+        for each position, enclosing the rectangular apertures for the
+        "exact" case.
+        """
 
-        return [(slice(ymin, ymax), slice(xmin, xmax))
-                for xmin, xmax, ymin, ymax in zip(x_min, x_max, y_min, y_max)]
+        w2 = self.w_out / 2.
+        h2 = self.h_out / 2.
+        cos_theta = math.cos(self.theta)
+        sin_theta = math.sin(self.theta)
+        dx1 = abs(w2 * cos_theta - h2 * sin_theta)
+        dy1 = abs(w2 * sin_theta + h2 * cos_theta)
+        dx2 = abs(w2 * cos_theta + h2 * sin_theta)
+        dy2 = abs(w2 * sin_theta - h2 * cos_theta)
+        dx = max(dx1, dx2)
+        dy = max(dy1, dy2)
+
+        xmin = self.positions[:, 0] - dx
+        xmax = self.positions[:, 0] + dx
+        ymin = self.positions[:, 1] - dy
+        ymax = self.positions[:, 1] + dy
+
+        return [BoundingBox._from_float(x0, x1, y0, y1)
+                for x0, x1, y0, y1 in zip(xmin, xmax, ymin, ymax)]
 
     def area(self):
         return self.w_out * self.h_out - self.w_in * self.h_in
@@ -323,10 +347,34 @@ class RectangularAnnulus(RectangularMaskMixin, PixelAperture):
             patch_outer = mpatches.Rectangle(positions_outer[i],
                                              self.w_out, self.h_out,
                                              theta_deg, **kwargs)
-            path = _make_annulus_path(patch_inner, patch_outer)
+            path = self._make_annulus_path(patch_inner, patch_outer)
             patch = mpatches.PathPatch(path, **kwargs)
             ax.add_patch(patch)
 
+    def to_sky(self, wcs, mode='all'):
+        """
+        Convert the aperture to a `SkyRectangularAnnulus` object
+        defined in celestial coordinates.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The world coordinate system (WCS) transformation to use.
+
+        mode : {'all', 'wcs'}, optional
+            Whether to do the transformation including distortions
+            (``'all'``; default) or only including only the core WCS
+            transformation (``'wcs'``).
+
+        Returns
+        -------
+        aperture : `SkyRectangularAnnulus` object
+            A `SkyRectangularAnnulus` object.
+        """
+
+        sky_params = self._to_sky_params(wcs, mode=mode)
+        return SkyRectangularAnnulus(**sky_params)
+
 
 class SkyRectangularAperture(SkyAperture):
     """
@@ -358,7 +406,7 @@ class SkyRectangularAperture(SkyAperture):
         if isinstance(positions, SkyCoord):
             self.positions = positions
         else:
-            raise TypeError("positions should be a SkyCoord instance")
+            raise TypeError('positions must be a SkyCoord instance')
 
         assert_angle_or_pixel('w', w)
         assert_angle_or_pixel('h', h)
@@ -371,16 +419,17 @@ class SkyRectangularAperture(SkyAperture):
         self.w = w
         self.h = h
         self.theta = theta
+        self._params = ['w', 'h', 'theta']
 
     def to_pixel(self, wcs, mode='all'):
         """
-        Convert the aperture to a `RectangularAperture` instance in
-        pixel coordinates.
+        Convert the aperture to a `RectangularAperture` object defined
+        in pixel coordinates.
 
         Parameters
         ----------
         wcs : `~astropy.wcs.WCS`
-            The WCS transformation to use.
+            The world coordinate system (WCS) transformation to use.
 
         mode : {'all', 'wcs'}, optional
             Whether to do the transformation including distortions
@@ -393,24 +442,8 @@ class SkyRectangularAperture(SkyAperture):
             A `RectangularAperture` object.
         """
 
-        x, y = skycoord_to_pixel(self.positions, wcs, mode=mode)
-        central_pos = SkyCoord([wcs.wcs.crval], frame=self.positions.name,
-                               unit=wcs.wcs.cunit)
-        xc, yc, scale, angle = skycoord_to_pixel_scale_angle(central_pos, wcs)
-
-        if self.w.unit.physical_type == 'angle':
-            w = (scale * self.w).to(u.pixel).value
-            h = (scale * self.h).to(u.pixel).value
-        else:
-            # pixels
-            w = self.w.value
-            h = self.h.value
-
-        theta = (angle + self.theta).to(u.radian).value
-
-        pixel_positions = np.array([x, y]).transpose()
-
-        return RectangularAperture(pixel_positions, w, h, theta)
+        pixel_params = self._to_pixel_params(wcs, mode=mode)
+        return RectangularAperture(**pixel_params)
 
 
 class SkyRectangularAnnulus(SkyAperture):
@@ -452,7 +485,8 @@ class SkyRectangularAnnulus(SkyAperture):
         if isinstance(positions, SkyCoord):
             self.positions = positions
         else:
-            raise TypeError("positions should be a SkyCoord instance")
+            raise TypeError('positions must be a SkyCoord instance')
+
         assert_angle_or_pixel('w_in', w_in)
         assert_angle_or_pixel('w_out', w_out)
         assert_angle_or_pixel('h_out', h_out)
@@ -470,16 +504,17 @@ class SkyRectangularAnnulus(SkyAperture):
         self.w_out = w_out
         self.h_out = h_out
         self.theta = theta
+        self._params = ['w_in', 'w_out', 'h_out', 'theta']
 
     def to_pixel(self, wcs, mode='all'):
         """
-        Convert the aperture to a `RectangularAnnulus` instance in pixel
-        coordinates.
+        Convert the aperture to a `RectangularAnnulus` object defined in
+        pixel coordinates.
 
         Parameters
         ----------
         wcs : `~astropy.wcs.WCS`
-            The WCS transformation to use.
+            The world coordinate system (WCS) transformation to use.
 
         mode : {'all', 'wcs'}, optional
             Whether to do the transformation including distortions
@@ -492,23 +527,5 @@ class SkyRectangularAnnulus(SkyAperture):
             A `RectangularAnnulus` object.
         """
 
-        x, y = skycoord_to_pixel(self.positions, wcs, mode=mode)
-        central_pos = SkyCoord([wcs.wcs.crval], frame=self.positions.name,
-                               unit=wcs.wcs.cunit)
-        xc, yc, scale, angle = skycoord_to_pixel_scale_angle(central_pos, wcs)
-
-        if self.w_in.unit.physical_type == 'angle':
-            w_in = (scale * self.w_in).to(u.pixel).value
-            w_out = (scale * self.w_out).to(u.pixel).value
-            h_out = (scale * self.h_out).to(u.pixel).value
-        else:
-            # pixels
-            w_in = self.w_in.value
-            w_out = self.w_out.value
-            h_out = self.h_out.value
-
-        theta = (angle + self.theta).to(u.radian).value
-
-        pixel_positions = np.array([x, y]).transpose()
-
-        return RectangularAnnulus(pixel_positions, w_in, w_out, h_out, theta)
+        pixel_params = self._to_pixel_params(wcs, mode=mode)
+        return RectangularAnnulus(**pixel_params)
