@@ -8,21 +8,31 @@ algorithms in aperture_photometry, not in the wrappers.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import pytest
 import numpy as np
 from numpy.testing import (assert_allclose, assert_array_equal,
                            assert_array_less)
+
+from astropy.coordinates import SkyCoord
 from astropy.io import fits
 from astropy.nddata import NDData
+from astropy.table import Table
+from astropy.tests.helper import remote_data
 import astropy.units as u
-from astropy.tests.helper import pytest, remote_data
+from astropy.wcs.utils import pixel_to_skycoord
 
-from ..core import *
-from ..circle import *
-from ..ellipse import *
-from ..rectangle import *
+from ..core import aperture_photometry
+from ..circle import (CircularAperture, CircularAnnulus, SkyCircularAperture,
+                      SkyCircularAnnulus)
+from ..ellipse import (EllipticalAperture, EllipticalAnnulus,
+                       SkyEllipticalAperture, SkyEllipticalAnnulus)
+from ..rectangle import (RectangularAperture, RectangularAnnulus,
+                         SkyRectangularAperture, SkyRectangularAnnulus)
+from ...datasets import (get_path, make_4gaussians_image, make_wcs,
+                         make_imagehdu)
 
 try:
-    import matplotlib
+    import matplotlib    # noqa
     HAS_MATPLOTLIB = True
 except ImportError:
     HAS_MATPLOTLIB = False
@@ -71,10 +81,9 @@ def test_inside_array_simple(aperture_class, params):
 @pytest.mark.skipif('not HAS_MATPLOTLIB')
 @pytest.mark.parametrize(('aperture_class', 'params'), TEST_APERTURES)
 def test_aperture_plots(aperture_class, params):
-    # This test should run without any errors, and there is no return value
-
-    # TODO for 0.2: check the content of the plot
-
+    # This test should run without any errors, and there is no return
+    # value.
+    # TODO: check the content of the plot
     aperture = aperture_class((20., 20.), *params)
     aperture.plot()
 
@@ -98,13 +107,15 @@ def test_aperture_pixel_positions():
 class BaseTestAperturePhotometry(object):
 
     def test_scalar_error(self):
-
         # Scalar error
         error = 1.
         if not hasattr(self, 'mask'):
             mask = None
+            true_error = np.sqrt(self.area)
         else:
             mask = self.mask
+            # 1 masked pixel
+            true_error = np.sqrt(self.area - 1)
 
         table1 = aperture_photometry(self.data,
                                      self.aperture, method='center',
@@ -124,7 +135,6 @@ class BaseTestAperturePhotometry(object):
                             atol=0.1)
         assert np.all(table1['aperture_sum'] < table3['aperture_sum'])
 
-        true_error = np.sqrt(self.area)
         if not isinstance(self.aperture, (RectangularAperture,
                                           RectangularAnnulus)):
             assert_allclose(table3['aperture_sum_err'], true_error)
@@ -133,7 +143,6 @@ class BaseTestAperturePhotometry(object):
         assert np.all(table1['aperture_sum_err'] < table3['aperture_sum_err'])
 
     def test_array_error(self):
-
         # Array error
         error = np.ones(self.data.shape, dtype=np.float)
         if not hasattr(self, 'mask'):
@@ -279,7 +288,7 @@ class TestMaskedSkipCircular(BaseTestAperturePhotometry):
     def setup_class(self):
         self.data = np.ones((40, 40), dtype=np.float)
         self.mask = np.zeros((40, 40), dtype=bool)
-        self.mask[19, 19] = True
+        self.mask[20, 20] = True
         position = (20., 20.)
         r = 10.
         self.aperture = CircularAperture(position, r)
@@ -290,7 +299,6 @@ class TestMaskedSkipCircular(BaseTestAperturePhotometry):
 class BaseTestDifferentData(object):
 
     def test_basic_circular_aperture_photometry(self):
-
         aperture = CircularAperture(self.position, self.radius)
         table = aperture_photometry(self.data, aperture,
                                     method='exact', unit='adu')
@@ -298,10 +306,10 @@ class BaseTestDifferentData(object):
         assert_allclose(table['aperture_sum'].value, self.true_flux)
         assert table['aperture_sum'].unit, self.fluxunit
 
-        #assert np.all(table['xcenter'] ==
-        #              np.transpose(self.position)[0])
-        #assert np.all(table['ycenter'] ==
-        #              np.transpose(self.position)[1])
+        assert np.all(table['xcenter'].value ==
+                      np.transpose(self.position)[0])
+        assert np.all(table['ycenter'].value ==
+                      np.transpose(self.position)[1])
 
 
 class TestInputPrimaryHDU(BaseTestDifferentData):
@@ -355,10 +363,6 @@ class TestInputNDData(BaseTestDifferentData):
 
 @remote_data
 def test_wcs_based_photometry_to_catalogue():
-    from astropy.coordinates import SkyCoord
-    from astropy.table import Table
-    from ...datasets import get_path
-
     pathcat = get_path('spitzer_example_catalog.xml', location='remote')
     pathhdu = get_path('spitzer_example_image.fits', location='remote')
     hdu = fits.open(pathhdu)
@@ -393,12 +397,9 @@ def test_wcs_based_photometry_to_catalogue():
 
 
 def test_wcs_based_photometry():
-    from astropy.wcs import WCS
-    from astropy.wcs.utils import pixel_to_skycoord
-    from ...datasets import make_4gaussians_image
-
-    hdu = make_4gaussians_image(hdu=True, wcs=True)
-    wcs = WCS(header=hdu.header)
+    data = make_4gaussians_image()
+    wcs = make_wcs(data.shape)
+    hdu = make_imagehdu(data, wcs=wcs)
 
     # hard wired positions in make_4gaussian_image
     pos_orig_pixel = u.Quantity(([160., 25., 150., 90.],
@@ -409,19 +410,19 @@ def test_wcs_based_photometry():
     pos_skycoord_s = pos_skycoord[2]
 
     photometry_skycoord_circ = aperture_photometry(
-        hdu, SkyCircularAperture(pos_skycoord, 3 * u.deg))
+        hdu, SkyCircularAperture(pos_skycoord, 3 * u.arcsec))
     photometry_skycoord_circ_2 = aperture_photometry(
-        hdu, SkyCircularAperture(pos_skycoord, 2 * u.deg))
+        hdu, SkyCircularAperture(pos_skycoord, 2 * u.arcsec))
     photometry_skycoord_circ_s = aperture_photometry(
-        hdu, SkyCircularAperture(pos_skycoord_s, 3 * u.deg))
+        hdu, SkyCircularAperture(pos_skycoord_s, 3 * u.arcsec))
 
     assert_allclose(photometry_skycoord_circ['aperture_sum'][2],
                     photometry_skycoord_circ_s['aperture_sum'])
 
     photometry_skycoord_circ_ann = aperture_photometry(
-        hdu, SkyCircularAnnulus(pos_skycoord, 2 * u.deg, 3 * u.deg))
+        hdu, SkyCircularAnnulus(pos_skycoord, 2 * u.arcsec, 3 * u.arcsec))
     photometry_skycoord_circ_ann_s = aperture_photometry(
-        hdu, SkyCircularAnnulus(pos_skycoord_s, 2 * u.deg, 3 * u.deg))
+        hdu, SkyCircularAnnulus(pos_skycoord_s, 2 * u.arcsec, 3 * u.arcsec))
 
     assert_allclose(photometry_skycoord_circ_ann['aperture_sum'][2],
                     photometry_skycoord_circ_ann_s['aperture_sum'])
@@ -431,20 +432,20 @@ def test_wcs_based_photometry():
                     photometry_skycoord_circ_2['aperture_sum'])
 
     photometry_skycoord_ell = aperture_photometry(
-        hdu, SkyEllipticalAperture(pos_skycoord,
-                                   3 * u.deg, 3.0001 * u.deg, 45 * u.deg))
+        hdu, SkyEllipticalAperture(pos_skycoord, 3 * u.arcsec,
+                                   3.0001 * u.arcsec, 45 * u.arcsec))
     photometry_skycoord_ell_2 = aperture_photometry(
-        hdu, SkyEllipticalAperture(pos_skycoord,
-                                   2 * u.deg, 2.0001 * u.deg, 45 * u.deg))
+        hdu, SkyEllipticalAperture(pos_skycoord, 2 * u.arcsec,
+                                   2.0001 * u.arcsec, 45 * u.arcsec))
     photometry_skycoord_ell_s = aperture_photometry(
-        hdu, SkyEllipticalAperture(pos_skycoord_s,
-                                   3 * u.deg, 3.0001 * u.deg, 45 * u.deg))
+        hdu, SkyEllipticalAperture(pos_skycoord_s, 3 * u.arcsec,
+                                   3.0001 * u.arcsec, 45 * u.arcsec))
     photometry_skycoord_ell_ann = aperture_photometry(
-        hdu, SkyEllipticalAnnulus(pos_skycoord, 2 * u.deg, 3 * u.deg,
-                                  3.0001 * u.deg, 45 * u.deg))
+        hdu, SkyEllipticalAnnulus(pos_skycoord, 2 * u.arcsec, 3 * u.arcsec,
+                                  3.0001 * u.arcsec, 45 * u.arcsec))
     photometry_skycoord_ell_ann_s = aperture_photometry(
-        hdu, SkyEllipticalAnnulus(pos_skycoord_s, 2 * u.deg, 3 * u.deg,
-                                  3.0001 * u.deg, 45 * u.deg))
+        hdu, SkyEllipticalAnnulus(pos_skycoord_s, 2 * u.arcsec, 3 * u.arcsec,
+                                  3.0001 * u.arcsec, 45 * u.arcsec))
 
     assert_allclose(photometry_skycoord_ell['aperture_sum'][2],
                     photometry_skycoord_ell_s['aperture_sum'])
@@ -461,26 +462,26 @@ def test_wcs_based_photometry():
 
     photometry_skycoord_rec = aperture_photometry(
         hdu, SkyRectangularAperture(pos_skycoord,
-                                    6 * u.deg, 6 * u.deg,
-                                    0 * u.deg),
+                                    6 * u.arcsec, 6 * u.arcsec,
+                                    0 * u.arcsec),
         method='subpixel', subpixels=20)
     photometry_skycoord_rec_4 = aperture_photometry(
         hdu, SkyRectangularAperture(pos_skycoord,
-                                    4 * u.deg, 4 * u.deg,
-                                    0 * u.deg),
+                                    4 * u.arcsec, 4 * u.arcsec,
+                                    0 * u.arcsec),
         method='subpixel', subpixels=20)
     photometry_skycoord_rec_s = aperture_photometry(
         hdu, SkyRectangularAperture(pos_skycoord_s,
-                                    6 * u.deg, 6 * u.deg,
-                                    0 * u.deg),
+                                    6 * u.arcsec, 6 * u.arcsec,
+                                    0 * u.arcsec),
         method='subpixel', subpixels=20)
     photometry_skycoord_rec_ann = aperture_photometry(
-        hdu, SkyRectangularAnnulus(pos_skycoord, 4 * u.deg, 6 * u.deg,
-                                   6 * u.deg, 0 * u.deg),
+        hdu, SkyRectangularAnnulus(pos_skycoord, 4 * u.arcsec, 6 * u.arcsec,
+                                   6 * u.arcsec, 0 * u.arcsec),
         method='subpixel', subpixels=20)
     photometry_skycoord_rec_ann_s = aperture_photometry(
-        hdu, SkyRectangularAnnulus(pos_skycoord_s, 4 * u.deg, 6 * u.deg,
-                                   6 * u.deg, 0 * u.deg),
+        hdu, SkyRectangularAnnulus(pos_skycoord_s, 4 * u.arcsec, 6 * u.arcsec,
+                                   6 * u.arcsec, 0 * u.arcsec),
         method='subpixel', subpixels=20)
 
     assert_allclose(photometry_skycoord_rec['aperture_sum'][2],
@@ -498,7 +499,6 @@ def test_wcs_based_photometry():
 
 
 def test_basic_circular_aperture_photometry_unit():
-
     data1 = np.ones((40, 40), dtype=np.float)
     data2 = u.Quantity(data1, unit=u.adu)
 
@@ -593,20 +593,238 @@ def test_nan_inf_mask(value):
     desired = (np.pi * radius**2) - 1
     assert_allclose(tbl['aperture_sum'], desired)
 
+
 def test_aperture_partial_overlap():
     data = np.ones((20, 20))
     error = np.ones((20, 20))
-    xypos= [(10, 10), (0, 0), (0, 19), (19, 0), (19, 19)]
+    xypos = [(10, 10), (0, 0), (0, 19), (19, 0), (19, 19)]
     r = 5.
     aper = CircularAperture(xypos, r=r)
     tbl = aperture_photometry(data, aper, error=error)
-    assert_allclose(tbl['aperture_sum'][0], np.pi * r **2)
-    assert_array_less(tbl['aperture_sum'][1:], np.pi * r **2)
+    assert_allclose(tbl['aperture_sum'][0], np.pi * r ** 2)
+    assert_array_less(tbl['aperture_sum'][1:], np.pi * r ** 2)
 
     unit = u.MJy / u.sr
     tbl = aperture_photometry(data * unit, aper, error=error * unit)
-    assert_allclose(tbl['aperture_sum'][0].value, np.pi * r **2)
-    assert_array_less(tbl['aperture_sum'][1:].value, np.pi * r **2)
-    assert_array_less(tbl['aperture_sum_err'][1:].value, np.pi * r **2)
+    assert_allclose(tbl['aperture_sum'][0].value, np.pi * r ** 2)
+    assert_array_less(tbl['aperture_sum'][1:].value, np.pi * r ** 2)
+    assert_array_less(tbl['aperture_sum_err'][1:].value, np.pi * r ** 2)
     assert tbl['aperture_sum'].unit == unit
     assert tbl['aperture_sum_err'].unit == unit
+
+
+def test_pixel_aperture_repr():
+    aper = CircularAperture((10, 20), r=3.0)
+    a_repr = '<CircularAperture([[10, 20]], r=3.0)>'
+    a_str = 'Aperture: CircularAperture\npositions: [[10, 20]]\nr: 3.0'
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = CircularAnnulus((10, 20), r_in=3.0, r_out=5.0)
+    a_repr = '<CircularAnnulus([[10, 20]], r_in=3.0, r_out=5.0)>'
+    a_str = ('Aperture: CircularAnnulus\npositions: [[10, 20]]\nr_in: 3.0\n'
+             'r_out: 5.0')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = EllipticalAperture((10, 20), a=5.0, b=3.0, theta=15.0)
+    a_repr = '<EllipticalAperture([[10, 20]], a=5.0, b=3.0, theta=15.0)>'
+    a_str = ('Aperture: EllipticalAperture\npositions: [[10, 20]]\n'
+             'a: 5.0\nb: 3.0\ntheta: 15.0')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = EllipticalAnnulus((10, 20), a_in=4.0, a_out=8.0, b_out=4.0,
+                             theta=15.0)
+    a_repr = ('<EllipticalAnnulus([[10, 20]], a_in=4.0, a_out=8.0, b_out='
+              '4.0, theta=15.0)>')
+    a_str = ('Aperture: EllipticalAnnulus\npositions: [[10, 20]]\na_in: '
+             '4.0\na_out: 8.0\nb_out: 4.0\ntheta: 15.0')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = RectangularAperture((10, 20), w=5.0, h=3.0, theta=15.0)
+    a_repr = '<RectangularAperture([[10, 20]], w=5.0, h=3.0, theta=15.0)>'
+    a_str = ('Aperture: RectangularAperture\npositions: [[10, 20]]\n'
+             'w: 5.0\nh: 3.0\ntheta: 15.0')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = RectangularAnnulus((10, 20), w_in=4.0, w_out=8.0, h_out=4.0,
+                              theta=15.0)
+    a_repr = ('<RectangularAnnulus([[10, 20]], w_in=4.0, w_out=8.0, '
+              'h_out=4.0, theta=15.0)>')
+    a_str = ('Aperture: RectangularAnnulus\npositions: [[10, 20]]\n'
+             'w_in: 4.0\nw_out: 8.0\nh_out: 4.0\ntheta: 15.0')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+
+def test_sky_aperture_repr():
+    s = SkyCoord([1, 2], [3, 4], unit='deg')
+
+    aper = SkyCircularAperture(s, r=3*u.pix)
+    a_repr = ('<SkyCircularAperture(<SkyCoord (ICRS): (ra, dec) in deg\n'
+              '    [( 1.,  3.), ( 2.,  4.)]>, r=3.0 pix)>')
+    a_str = ('Aperture: SkyCircularAperture\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'r: 3.0 pix')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = SkyCircularAnnulus(s, r_in=3.*u.pix, r_out=5*u.pix)
+    a_repr = ('<SkyCircularAnnulus(<SkyCoord (ICRS): (ra, dec) in deg\n'
+              '    [( 1.,  3.), ( 2.,  4.)]>, r_in=3.0 pix, r_out=5.0 pix)>')
+    a_str = ('Aperture: SkyCircularAnnulus\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'r_in: 3.0 pix\nr_out: 5.0 pix')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = SkyEllipticalAperture(s, a=3*u.pix, b=5*u.pix, theta=15*u.deg)
+    a_repr = ('<SkyEllipticalAperture(<SkyCoord (ICRS): (ra, dec) in '
+              'deg\n    [( 1.,  3.), ( 2.,  4.)]>, a=3.0 pix, b=5.0 pix,'
+              ' theta=15.0 deg)>')
+    a_str = ('Aperture: SkyEllipticalAperture\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'a: 3.0 pix\nb: 5.0 pix\ntheta: 15.0 deg')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = SkyEllipticalAnnulus(s, a_in=3*u.pix, a_out=5*u.pix, b_out=3*u.pix,
+                                theta=15*u.deg)
+    a_repr = ('<SkyEllipticalAnnulus(<SkyCoord (ICRS): (ra, dec) in '
+              'deg\n    [( 1.,  3.), ( 2.,  4.)]>, a_in=3.0 pix, '
+              'a_out=5.0 pix, b_out=3.0 pix, theta=15.0 deg)>')
+    a_str = ('Aperture: SkyEllipticalAnnulus\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'a_in: 3.0 pix\na_out: 5.0 pix\nb_out: 3.0 pix\n'
+             'theta: 15.0 deg')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = SkyRectangularAperture(s, w=3*u.pix, h=5*u.pix, theta=15*u.deg)
+    a_repr = ('<SkyRectangularAperture(<SkyCoord (ICRS): (ra, dec) in '
+              'deg\n    [( 1.,  3.), ( 2.,  4.)]>, w=3.0 pix, h=5.0 pix'
+              ', theta=15.0 deg)>')
+    a_str = ('Aperture: SkyRectangularAperture\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'w: 3.0 pix\nh: 5.0 pix\ntheta: 15.0 deg')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+    aper = SkyRectangularAnnulus(s, w_in=3*u.pix, w_out=3.4*u.pix,
+                                 h_out=5*u.pix, theta=15*u.deg)
+    a_repr = ('<SkyRectangularAnnulus(<SkyCoord (ICRS): (ra, dec) in deg'
+              '\n    [( 1.,  3.), ( 2.,  4.)]>, w_in=3.0 pix, '
+              'w_out=3.4 pix, h_out=5.0 pix, theta=15.0 deg)>')
+    a_str = ('Aperture: SkyRectangularAnnulus\npositions: <SkyCoord '
+             '(ICRS): (ra, dec) in deg\n    [( 1.,  3.), ( 2.,  4.)]>\n'
+             'w_in: 3.0 pix\nw_out: 3.4 pix\nh_out: 5.0 pix\n'
+             'theta: 15.0 deg')
+    assert repr(aper) == a_repr
+    assert str(aper) == a_str
+
+
+def test_rectangular_bbox():
+    # odd sizes
+    width = 7
+    height = 3
+    a = RectangularAperture((50, 50), w=width, h=height, theta=0)
+    assert a.bounding_boxes[0].shape == (height, width)
+
+    a = RectangularAperture((50.5, 50.5), w=width, h=height, theta=0)
+    assert a.bounding_boxes[0].shape == (height + 1, width + 1)
+
+    a = RectangularAperture((50, 50), w=width, h=height, theta=90.*np.pi/180.)
+    assert a.bounding_boxes[0].shape == (width, height)
+
+    # even sizes
+    width = 8
+    height = 4
+    a = RectangularAperture((50, 50), w=width, h=height, theta=0)
+    assert a.bounding_boxes[0].shape == (height + 1, width + 1)
+
+    a = RectangularAperture((50.5, 50.5), w=width, h=height, theta=0)
+    assert a.bounding_boxes[0].shape == (height, width)
+
+    a = RectangularAperture((50.5, 50.5), w=width, h=height,
+                            theta=90.*np.pi/180.)
+    assert a.bounding_boxes[0].shape == (width, height)
+
+
+def test_elliptical_bbox():
+    # integer axes
+    a = 7
+    b = 3
+    ap = EllipticalAperture((50, 50), a=a, b=b, theta=0)
+    assert ap.bounding_boxes[0].shape == (2*b + 1, 2*a + 1)
+
+    ap = EllipticalAperture((50.5, 50.5), a=a, b=b, theta=0)
+    assert ap.bounding_boxes[0].shape == (2*b, 2*a)
+
+    ap = EllipticalAperture((50, 50), a=a, b=b, theta=90.*np.pi/180.)
+    assert ap.bounding_boxes[0].shape == (2*a + 1, 2*b + 1)
+
+    # fractional axes
+    a = 7.5
+    b = 4.5
+    ap = EllipticalAperture((50, 50), a=a, b=b, theta=0)
+    assert ap.bounding_boxes[0].shape == (2*b, 2*a)
+
+    ap = EllipticalAperture((50.5, 50.5), a=a, b=b, theta=0)
+    assert ap.bounding_boxes[0].shape == (2*b + 1, 2*a + 1)
+
+    ap = EllipticalAperture((50, 50), a=a, b=b, theta=90.*np.pi/180.)
+    assert ap.bounding_boxes[0].shape == (2*a, 2*b)
+
+
+def test_to_sky_pixel():
+    data = make_4gaussians_image()
+    wcs = make_wcs(data.shape)
+
+    ap = CircularAperture(((12.3, 15.7), (48.19, 98.14)), r=3.14)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.r, ap2.r)
+
+    ap = CircularAnnulus(((12.3, 15.7), (48.19, 98.14)), r_in=3.14,
+                         r_out=5.32)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.r_in, ap2.r_in)
+    assert_allclose(ap.r_out, ap2.r_out)
+
+    ap = EllipticalAperture(((12.3, 15.7), (48.19, 98.14)), a=3.14, b=5.32,
+                            theta=103.*np.pi/180.)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.a, ap2.a)
+    assert_allclose(ap.b, ap2.b)
+    assert_allclose(ap.theta, ap2.theta)
+
+    ap = EllipticalAnnulus(((12.3, 15.7), (48.19, 98.14)), a_in=3.14,
+                           a_out=15.32, b_out=4.89, theta=103.*np.pi/180.)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.a_in, ap2.a_in)
+    assert_allclose(ap.a_out, ap2.a_out)
+    assert_allclose(ap.b_out, ap2.b_out)
+    assert_allclose(ap.theta, ap2.theta)
+
+    ap = RectangularAperture(((12.3, 15.7), (48.19, 98.14)), w=3.14, h=5.32,
+                             theta=103.*np.pi/180.)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.w, ap2.w)
+    assert_allclose(ap.h, ap2.h)
+    assert_allclose(ap.theta, ap2.theta)
+
+    ap = RectangularAnnulus(((12.3, 15.7), (48.19, 98.14)), w_in=3.14,
+                            w_out=15.32, h_out=4.89, theta=103.*np.pi/180.)
+    ap2 = ap.to_sky(wcs).to_pixel(wcs)
+    assert_allclose(ap.positions, ap2.positions)
+    assert_allclose(ap.w_in, ap2.w_in)
+    assert_allclose(ap.w_out, ap2.w_out)
+    assert_allclose(ap.h_out, ap2.h_out)
+    assert_allclose(ap.theta, ap2.theta)
