@@ -10,19 +10,18 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import abc
 
+import six
 import numpy as np
-from astropy.extern import six
+from astropy.stats import (SigmaClip, biweight_location, biweight_scale,
+                           mad_std)
 from astropy.utils.misc import InheritDocstrings
 
-from ..extern.sigma_clipping import sigma_clip
-from ..extern.stats import mad_std, biweight_location, biweight_midvariance
 
-
-__all__ = ['SigmaClip', 'BackgroundBase', 'BackgroundRMSBase',
-           'MeanBackground', 'MedianBackground', 'ModeEstimatorBackground',
+__all__ = ['BackgroundBase', 'BackgroundRMSBase', 'MeanBackground',
+           'MedianBackground', 'ModeEstimatorBackground',
            'MMMBackground', 'SExtractorBackground',
            'BiweightLocationBackground', 'StdBackgroundRMS',
-           'MADStdBackgroundRMS', 'BiweightMidvarianceBackgroundRMS']
+           'MADStdBackgroundRMS', 'BiweightScaleBackgroundRMS']
 
 
 def _masked_median(data, axis=None):
@@ -62,90 +61,6 @@ class _ABCMetaAndInheritDocstrings(InheritDocstrings, abc.ABCMeta):
     pass
 
 
-class SigmaClip(object):
-    """
-    Class to perform sigma clipping.
-
-    Parameters
-    ----------
-    sigma : float, optional
-        The number of standard deviations to use for both the lower and
-        upper clipping limit. These limits are overridden by
-        ``sigma_lower`` and ``sigma_upper``, if input. Defaults to 3.
-    sigma_lower : float or `None`, optional
-        The number of standard deviations to use as the lower bound for
-        the clipping limit. If `None` then the value of ``sigma`` is
-        used. Defaults to `None`.
-    sigma_upper : float or `None`, optional
-        The number of standard deviations to use as the upper bound for
-        the clipping limit. If `None` then the value of ``sigma`` is
-        used. Defaults to `None`.
-    iters : int or `None`, optional
-        The number of iterations to perform sigma clipping, or `None` to
-        clip until convergence is achieved (i.e., continue until the
-        last iteration clips nothing). Defaults to 5.
-    cenfunc : callable, optional
-        The function used to compute the center for the clipping. Must
-        be a callable that takes in a masked array and outputs the
-        central value. Defaults to the median (`numpy.ma.median`).
-    stdfunc : callable, optional
-        The function used to compute the standard deviation about the
-        center. Must be a callable that takes in a masked array and
-        outputs a width estimator. Masked (rejected) pixels are those
-        where::
-
-             deviation < (-sigma_lower * stdfunc(deviation))
-             deviation > (sigma_upper * stdfunc(deviation))
-
-        where::
-
-            deviation = data - cenfunc(data [,axis=int])
-
-        Defaults to the standard deviation (`numpy.std`).
-    """
-
-    def __init__(self, sigma=3., sigma_lower=None, sigma_upper=None, iters=5,
-                 cenfunc=np.ma.median, stdfunc=np.std):
-        self.sigma = sigma
-        self.sigma_lower = sigma_lower
-        self.sigma_upper = sigma_upper
-        self.iters = iters
-        self.cenfunc = np.ma.median
-        self.stdfunc = np.std
-
-    def __call__(self, data, axis=None, copy=True):
-        """
-        Perform sigma clipping on the provided data.
-
-        Parameters
-        ----------
-        data : array-like
-            The data to be sigma clipped.
-        axis : int or `None`, optional
-            If not `None`, clip along the given axis.  For this case,
-            ``axis`` will be passed on to ``cenfunc`` and ``stdfunc``,
-            which are expected to return an array with the axis
-            dimension removed (like the numpy functions).  If `None`,
-            clip over all axes.  Defaults to `None`.
-        copy : bool, optional
-            If `True`, the ``data`` array will be copied.  If `False`,
-            the returned masked array data will contain the same array
-            as ``data``.  Defaults to `True`.
-
-        Returns
-        -------
-        filtered_data : `numpy.ma.MaskedArray`
-            A masked array with the same shape as ``data`` input, where
-            the points rejected by the algorithm have been masked.
-        """
-
-        return sigma_clip(data, sigma=self.sigma,
-                          sigma_lower=self.sigma_lower,
-                          sigma_upper=self.sigma_upper, iters=self.iters,
-                          cenfunc=self.cenfunc, stdfunc=self.stdfunc,
-                          axis=axis, copy=copy)
-
-
 @six.add_metaclass(_ABCMetaAndInheritDocstrings)
 class BackgroundBase(object):
     """
@@ -153,10 +68,11 @@ class BackgroundBase(object):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
     """
 
     def __init__(self, sigma_clip=SigmaClip(sigma=3., iters=5)):
@@ -194,10 +110,11 @@ class BackgroundRMSBase(object):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
     """
 
     def __init__(self, sigma_clip=SigmaClip(sigma=3., iters=5)):
@@ -235,14 +152,16 @@ class MeanBackground(BackgroundBase):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, MeanBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import MeanBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = MeanBackground(sigma_clip)
@@ -276,14 +195,16 @@ class MedianBackground(BackgroundBase):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, MedianBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import MedianBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = MedianBackground(sigma_clip)
@@ -321,14 +242,16 @@ class ModeEstimatorBackground(BackgroundBase):
         The multiplicative factor for the data median.  Defaults to 3.
     mean_factor : float, optional
         The multiplicative factor for the data mean.  Defaults to 2.
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, ModeEstimatorBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import ModeEstimatorBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = ModeEstimatorBackground(median_factor=3., mean_factor=2.,
@@ -371,14 +294,16 @@ class MMMBackground(ModeEstimatorBackground):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, MMMBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import MMMBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = MMMBackground(sigma_clip=sigma_clip)
@@ -421,14 +346,16 @@ class SExtractorBackground(BackgroundBase):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, SExtractorBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import SExtractorBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = SExtractorBackground(sigma_clip)
@@ -483,14 +410,16 @@ class BiweightLocationBackground(BackgroundBase):
     M : float, optional
         Initial guess for the biweight location.  Default value is
         `None`.
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, BiweightLocationBackground
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import BiweightLocationBackground
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkg = BiweightLocationBackground(sigma_clip=sigma_clip)
@@ -529,14 +458,16 @@ class StdBackgroundRMS(BackgroundRMSBase):
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, StdBackgroundRMS
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import StdBackgroundRMS
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkgrms = StdBackgroundRMS(sigma_clip)
@@ -573,22 +504,24 @@ class MADStdBackgroundRMS(BackgroundRMSBase):
 
     .. math::
 
-        \\sigma \\approx \\frac{{\\textrm{{MAD}}}}{{\Phi^{{-1}}(3/4)}}
-            \\approx 1.4826 \ \\textrm{{MAD}}
+        \\sigma \\approx \\frac{{\\textrm{{MAD}}}}{{\\Phi^{{-1}}(3/4)}}
+            \\approx 1.4826 \\ \\textrm{{MAD}}
 
-    where :math:`\Phi^{{-1}}(P)` is the normal inverse cumulative
+    where :math:`\\Phi^{{-1}}(P)` is the normal inverse cumulative
     distribution function evaluated at probability :math:`P = 3/4`.
 
     Parameters
     ----------
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, MADStdBackgroundRMS
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import MADStdBackgroundRMS
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
     >>> bkgrms = MADStdBackgroundRMS(sigma_clip)
@@ -615,10 +548,10 @@ class MADStdBackgroundRMS(BackgroundRMSBase):
         return mad_std(data, axis=axis)
 
 
-class BiweightMidvarianceBackgroundRMS(BackgroundRMSBase):
+class BiweightScaleBackgroundRMS(BackgroundRMSBase):
     """
     Class to calculate the background RMS in an array as the
-    (sigma-clipped) biweight midvariance.
+    (sigma-clipped) biweight scale.
 
     Parameters
     ----------
@@ -628,17 +561,19 @@ class BiweightMidvarianceBackgroundRMS(BackgroundRMSBase):
     M : float, optional
         Initial guess for the biweight location.  Default value is
         `None`.
-    sigma_clip : `SigmaClip` object, optional
-        A `SigmaClip` object that defines the sigma clipping parameters.
-        If `None` then no sigma clipping will be performed.  The default
-        is to perform sigma clipping with ``sigma=3.`` and ``iters=5``.
+    sigma_clip : `astropy.stats.SigmaClip` object, optional
+        A `~astropy.stats.SigmaClip` object that defines the sigma
+        clipping parameters.  If `None` then no sigma clipping will be
+        performed.  The default is to perform sigma clipping with
+        ``sigma=3.`` and ``iters=5``.
 
     Examples
     --------
-    >>> from photutils import SigmaClip, BiweightMidvarianceBackgroundRMS
+    >>> from astropy.stats import SigmaClip
+    >>> from photutils import BiweightScaleBackgroundRMS
     >>> data = np.arange(100)
     >>> sigma_clip = SigmaClip(sigma=3.)
-    >>> bkgrms = BiweightMidvarianceBackgroundRMS(sigma_clip=sigma_clip)
+    >>> bkgrms = BiweightScaleBackgroundRMS(sigma_clip=sigma_clip)
 
     The background RMS value can be calculated by using the
     `calc_background_rms` method, e.g.:
@@ -656,7 +591,7 @@ class BiweightMidvarianceBackgroundRMS(BackgroundRMSBase):
     """
 
     def __init__(self, c=9.0, M=None, **kwargs):
-        super(BiweightMidvarianceBackgroundRMS, self).__init__(**kwargs)
+        super(BiweightScaleBackgroundRMS, self).__init__(**kwargs)
         self.c = c
         self.M = M
 
@@ -664,4 +599,4 @@ class BiweightMidvarianceBackgroundRMS(BackgroundRMSBase):
         if self.sigma_clip is not None:
             data = self.sigma_clip(data, axis=axis)
 
-        return biweight_midvariance(data, c=self.c, M=self.M, axis=axis)
+        return biweight_scale(data, c=self.c, M=self.M, axis=axis)
