@@ -4,19 +4,25 @@ from __future__ import (absolute_import, division, print_function,
 import itertools
 
 import numpy as np
-from numpy.testing import assert_allclose
-from astropy.tests.helper import pytest
+from numpy.testing import assert_allclose, assert_equal
+import pytest
 
 from ..core import MeanBackground
 from ..background_2d import (BkgZoomInterpolator, BkgIDWInterpolator,
                              Background2D)
 
 try:
-    import scipy
+    import scipy    # noqa
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
 
+try:
+    import matplotlib    # noqa
+    HAS_MATPLOTLIB = True
+except ImportError:
+    HAS_MATPLOTLIB = False
+
 
 DATA = np.ones((100, 100))
 BKG_RMS = np.zeros((100, 100))
@@ -25,7 +31,6 @@ BKG_RMS_MESH = np.zeros((4, 4))
 PADBKG_MESH = np.ones((5, 5))
 PADBKG_RMS_MESH = np.zeros((5, 5))
 FILTER_SIZES = [(1, 1), (3, 3)]
-EDGE_METHODS = ['pad', 'crop']
 INTERPOLATORS = [BkgZoomInterpolator(), BkgIDWInterpolator()]
 
 
@@ -94,23 +99,34 @@ class TestBackground2D(object):
         assert_allclose(b.background, DATA)
         assert_allclose(b.background_rms, BKG_RMS)
 
-        # test mask padding
+        # test edge crop with
         b2 = Background2D(data, box_size, filter_size=(1, 1), mask=mask,
-                          bkg_estimator=MeanBackground(), edge_method='pad')
+                          bkg_estimator=MeanBackground(), edge_method='crop')
         assert_allclose(b2.background, DATA)
 
-    @pytest.mark.parametrize('exclude_mesh_method',
-                             (['any', 'all', 'threshold']))
-    def test_exclude_mesh(self, exclude_mesh_method):
-        b = Background2D(DATA, (25, 25),
-                         exclude_mesh_method=exclude_mesh_method)
-        assert_allclose(b.background, DATA)
+    def test_mask(self):
+        data = np.copy(DATA)
+        data[25:50, 25:50] = 100.
+        mask = np.zeros_like(DATA, dtype=np.bool)
+        mask[25:50, 25:50] = True
+        b1 = Background2D(data, (25, 25), filter_size=(1, 1), mask=None,
+                          bkg_estimator=MeanBackground())
 
-        # test if data is completely masked
+        assert_equal(b1.background_mesh, b1.background_mesh_ma)
+        assert_equal(b1.background_rms_mesh, b1.background_rms_mesh_ma)
+        assert not np.ma.is_masked(b1.mesh_nmasked)
+
+        b2 = Background2D(data, (25, 25), filter_size=(1, 1), mask=mask,
+                          bkg_estimator=MeanBackground())
+
+        assert np.ma.count(b2.background_mesh_ma) < b2.nboxes
+        assert np.ma.count(b2.background_rms_mesh_ma) < b2.nboxes
+        assert np.ma.is_masked(b2.mesh_nmasked)
+
+    def test_completely_masked(self):
         with pytest.raises(ValueError):
             mask = np.ones_like(DATA, dtype=np.bool)
-            Background2D(DATA, (25, 25), mask=mask,
-                         exclude_mesh_method=exclude_mesh_method)
+            Background2D(DATA, (25, 25), mask=mask)
 
     def test_zero_padding(self):
         """Test case where padding is added only on one axis."""
@@ -162,10 +178,12 @@ class TestBackground2D(object):
         assert_allclose(b1.background, b2.background)
         assert_allclose(b1.background_rms, b2.background_rms)
 
-    def test_exclude_mesh_percentile(self):
+    def test_exclude_percentile(self):
         with pytest.raises(ValueError):
-            Background2D(DATA, (5, 5), exclude_mesh_method='threshold',
-                         exclude_mesh_percentile=101)
+            Background2D(DATA, (5, 5), exclude_percentile=-1)
+
+        with pytest.raises(ValueError):
+            Background2D(DATA, (5, 5), exclude_percentile=101)
 
     def test_mask_badshape(self):
         with pytest.raises(ValueError):
@@ -177,16 +195,12 @@ class TestBackground2D(object):
             Background2D(DATA, (23, 22), filter_size=(1, 1),
                          edge_method='not_valid')
 
-    def test_invalid_exclude_mesh_method(self):
-        with pytest.raises(ValueError):
-            Background2D(DATA, (23, 22), filter_size=(1, 1),
-                         exclude_mesh_method='not_valid')
-
     def test_invalid_mesh_idx_len(self):
         with pytest.raises(ValueError):
             bkg = Background2D(DATA, (25, 25), filter_size=(1, 1))
             bkg._make_2d_array(np.arange(3))
 
+    @pytest.mark.skipif('not HAS_MATPLOTLIB')
     def test_plot_meshes(self):
         """
         This test should run without any errors, but there is no return
