@@ -2,16 +2,17 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import pytest
 import numpy as np
 from numpy.testing import assert_allclose
-from astropy.tests.helper import pytest
+from astropy.stats import SigmaClip
 
 from ...datasets.make import make_noise_image
-from ..core import (SigmaClip, MeanBackground, MedianBackground,
+from ..core import (MeanBackground, MedianBackground,
                     ModeEstimatorBackground, MMMBackground,
                     SExtractorBackground, BiweightLocationBackground,
                     StdBackgroundRMS, MADStdBackgroundRMS,
-                    BiweightMidvarianceBackgroundRMS)
+                    BiweightScaleBackgroundRMS)
 
 
 BKG = 0.0
@@ -26,7 +27,7 @@ BKG_CLASS0 = [MeanBackground, MedianBackground, ModeEstimatorBackground,
 BKG_CLASS = BKG_CLASS0 + [BiweightLocationBackground]
 
 RMS_CLASS = [StdBackgroundRMS, MADStdBackgroundRMS,
-             BiweightMidvarianceBackgroundRMS]
+             BiweightScaleBackgroundRMS]
 
 SIGMA_CLIP = SigmaClip(sigma=3.)
 
