@@ -3,29 +3,23 @@
 
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
-from distutils.version import LooseVersion
+
 import numpy as np
-from astropy.table import Column, Table
 from astropy.stats import sigma_clipped_stats
-from ..segmentation import SegmentationImage
-from ..morphology import cutout_footprint, fit_2dgaussian
-from ..utils.convolution import _convolve_data
+from astropy.table import Column, Table
+
+from ..utils.cutouts import cutout_footprint
 from ..utils.wcs_helpers import pixel_to_icrs_coords
 
-import astropy
-if LooseVersion(astropy.__version__) < LooseVersion('1.1'):
-    ASTROPY_LT_1P1 = True
-else:
-    ASTROPY_LT_1P1 = False
 
-
-__all__ = ['detect_threshold', 'detect_sources', 'find_peaks']
+__all__ = ['detect_threshold', 'find_peaks']
 
 
 def detect_threshold(data, snr, background=None, error=None, mask=None,
                      mask_value=None, sigclip_sigma=3.0, sigclip_iters=None):
     """
-    Calculate a pixel-wise threshold image to be used to detect sources.
+    Calculate a pixel-wise threshold image that can be used to detect
+    sources.
 
     Parameters
     ----------
@@ -82,7 +76,7 @@ def detect_threshold(data, snr, background=None, error=None, mask=None,
 
     See Also
     --------
-    detect_sources
+    :func:`photutils.segmentation.detect_sources`
 
     Notes
     -----
@@ -95,15 +89,9 @@ def detect_threshold(data, snr, background=None, error=None, mask=None,
     """
 
     if background is None or error is None:
-        # TODO: remove when astropy 1.1 is released
-        if ASTROPY_LT_1P1:
-            data_mean, data_median, data_std = sigma_clipped_stats(
-                data, mask=mask, mask_val=mask_value, sigma=sigclip_sigma,
-                iters=sigclip_iters)
-        else:
-            data_mean, data_median, data_std = sigma_clipped_stats(
-                data, mask=mask, mask_value=mask_value, sigma=sigclip_sigma,
-                iters=sigclip_iters)
+        data_mean, data_median, data_std = sigma_clipped_stats(
+            data, mask=mask, mask_value=mask_value, sigma=sigclip_sigma,
+            iters=sigclip_iters)
         bkgrd_image = np.zeros_like(data) + data_mean
         bkgrdrms_image = np.zeros_like(data) + data_std
 
@@ -132,138 +120,6 @@ def detect_threshold(data, snr, background=None, error=None, mask=None,
     return background + (error * snr)
 
 
-def detect_sources(data, threshold, npixels, filter_kernel=None,
-                   connectivity=8):
-    """
-    Detect sources above a specified threshold value in an image and
-    return a `~photutils.segmentation.SegmentationImage` object.
-
-    Detected sources must have ``npixels`` connected pixels that are
-    each greater than the ``threshold`` value.  If the filtering option
-    is used, then the ``threshold`` is applied to the filtered image.
-
-    This function does not deblend overlapping sources.  First use this
-    function to detect sources followed by
-    :func:`~photutils.detection.deblend_sources` to deblend sources.
-
-    Parameters
-    ----------
-    data : array_like
-        The 2D array of the image.
-
-    threshold : float or array-like
-        The data value or pixel-wise data values to be used for the
-        detection threshold.  A 2D ``threshold`` must have the same
-        shape as ``data``.  See `detect_threshold` for one way to create
-        a ``threshold`` image.
-
-    npixels : int
-        The number of connected pixels, each greater than ``threshold``,
-        that an object must have to be detected.  ``npixels`` must be a
-        positive integer.
-
-    filter_kernel : array-like (2D) or `~astropy.convolution.Kernel2D`, optional
-        The 2D array of the kernel used to filter the image before
-        thresholding.  Filtering the image will smooth the noise and
-        maximize detectability of objects with a shape similar to the
-        kernel.
-
-    connectivity : {4, 8}, optional
-        The type of pixel connectivity used in determining how pixels
-        are grouped into a detected source.  The options are 4 or 8
-        (default).  4-connected pixels touch along their edges.
-        8-connected pixels touch along their edges or corners.  For
-        reference, SExtractor uses 8-connected pixels.
-
-    Returns
-    -------
-    segment_image : `~photutils.segmentation.SegmentationImage`
-        A 2D segmentation image, with the same shape as ``data``, where
-        sources are marked by different positive integer values.  A
-        value of zero is reserved for the background.
-
-    See Also
-    --------
-    detect_threshold, :class:`photutils.segmentation.SegmentationImage`,
-    :func:`photutils.segmentation.source_properties`
-    :func:`photutils.detection.deblend_sources`
-
-    Examples
-    --------
-
-    .. plot::
-        :include-source:
-
-        # make a table of Gaussian sources
-        from astropy.table import Table
-        table = Table()
-        table['amplitude'] = [50, 70, 150, 210]
-        table['x_mean'] = [160, 25, 150, 90]
-        table['y_mean'] = [70, 40, 25, 60]
-        table['x_stddev'] = [15.2, 5.1, 3., 8.1]
-        table['y_stddev'] = [2.6, 2.5, 3., 4.7]
-        table['theta'] = np.array([145., 20., 0., 60.]) * np.pi / 180.
-
-        # make an image of the sources with Gaussian noise
-        from photutils.datasets import make_gaussian_sources
-        from photutils.datasets import make_noise_image
-        shape = (100, 200)
-        sources = make_gaussian_sources(shape, table)
-        noise = make_noise_image(shape, type='gaussian', mean=0.,
-                                 stddev=5., random_state=12345)
-        image = sources + noise
-
-        # detect the sources
-        from photutils import detect_threshold, detect_sources
-        threshold = detect_threshold(image, snr=3)
-        from astropy.convolution import Gaussian2DKernel
-        sigma = 3.0 / (2.0 * np.sqrt(2.0 * np.log(2.0)))   # FWHM = 3
-        kernel = Gaussian2DKernel(sigma, x_size=3, y_size=3)
-        kernel.normalize()
-        segm = detect_sources(image, threshold, npixels=5,
-                              filter_kernel=kernel)
-
-        # plot the image and the segmentation image
-        import matplotlib.pyplot as plt
-        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(8, 8))
-        ax1.imshow(image, origin='lower', interpolation='nearest')
-        ax2.imshow(segm.data, origin='lower', interpolation='nearest')
-    """
-
-    from scipy import ndimage
-
-    if (npixels <= 0) or (int(npixels) != npixels):
-        raise ValueError('npixels must be a positive integer, got '
-                         '"{0}"'.format(npixels))
-
-    image = (_convolve_data(
-        data, filter_kernel, mode='constant', fill_value=0.0,
-        check_normalization=True) > threshold)
-
-    if connectivity == 4:
-        selem = ndimage.generate_binary_structure(2, 1)
-    elif connectivity == 8:
-        selem = ndimage.generate_binary_structure(2, 2)
-    else:
-        raise ValueError('Invalid connectivity={0}.  '
-                         'Options are 4 or 8'.format(connectivity))
-
-    objlabels, nobj = ndimage.label(image, structure=selem)
-    objslices = ndimage.find_objects(objlabels)
-
-    # remove objects with less than npixels
-    for objslice in objslices:
-        objlabel = objlabels[objslice]
-        obj_npix = len(np.where(objlabel.ravel() != 0)[0])
-        if obj_npix < npixels:
-            objlabels[objslice] = 0
-
-    # relabel to make sequential label indices
-    objlabels, nobj = ndimage.label(objlabels, structure=selem)
-
-    return SegmentationImage(objlabels)
-
-
 def find_peaks(data, threshold, box_size=3, footprint=None, mask=None,
                border_width=None, npeaks=np.inf, subpixel=False, error=None,
                wcs=None):
@@ -393,6 +249,8 @@ def find_peaks(data, threshold, box_size=3, footprint=None, mask=None,
         peak_values = peak_values[idx]
 
     if subpixel:
+        from ..centroids import fit_2dgaussian    # prevents circular import
+
         x_centroid, y_centroid = [], []
         fit_peak_values = []
         for (y_peak, x_peak) in zip(y_peaks, x_peaks):
@@ -403,10 +261,10 @@ def find_peaks(data, threshold, box_size=3, footprint=None, mask=None,
             if gaussian_fit is None:
                 x_cen, y_cen, fit_peak_value = np.nan, np.nan, np.nan
             else:
-                x_cen = slc[1].start + gaussian_fit.x_mean_1.value
-                y_cen = slc[0].start + gaussian_fit.y_mean_1.value
-                fit_peak_value = (gaussian_fit.amplitude_0.value +
-                                  gaussian_fit.amplitude_1.value)
+                x_cen = slc[1].start + gaussian_fit.x_mean.value
+                y_cen = slc[0].start + gaussian_fit.y_mean.value
+                fit_peak_value = (gaussian_fit.constant.value +
+                                  gaussian_fit.amplitude.value)
             x_centroid.append(x_cen)
             y_centroid.append(y_cen)
             fit_peak_values.append(fit_peak_value)
