@@ -1,28 +1,116 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""Functions for detecting stars in an astronomical image."""
+"""
+This module implements classes, called Finders, for detecting stars in
+an astronomical image. The convention is that all Finders are subclasses
+of an abstract class called ``StarFinderBase``.  Each Finder class
+should define a method called ``find_stars`` that finds stars in an
+image.
+"""
 
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 from collections import defaultdict
 import warnings
 import math
+import abc
+
+import six
 import numpy as np
 from astropy.table import Column, Table
 from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.misc import InheritDocstrings
 from astropy.stats import gaussian_fwhm_to_sigma
-from .core import _convolve_data, find_peaks
+
+from .core import find_peaks
+from ..utils.convolution import filter_data
 
 
-__all__ = ['daofind', 'irafstarfind']
+__all__ = ['DAOStarFinder', 'IRAFStarFinder', 'StarFinderBase']
 
 
-def daofind(data, threshold, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5,
-            sharplo=0.2, sharphi=1.0, roundlo=-1.0, roundhi=1.0, sky=0.0,
-            exclude_border=False):
+class _ABCMetaAndInheritDocstrings(InheritDocstrings, abc.ABCMeta):
+    pass
+
+
+@six.add_metaclass(_ABCMetaAndInheritDocstrings)
+class StarFinderBase(object):
+    """
+    Abstract base class for Star Finders.
     """
-    Detect stars in an image using the DAOFIND algorithm.
 
-    `DAOFIND`_ (`Stetson 1987; PASP 99, 191
+    def __call__(self, data):
+        return self.find_stars(data)
+
+    @abc.abstractmethod
+    def find_stars(self, data):
+        """
+        Find stars in an astronomical image.
+
+        Parameters
+        ----------
+        data : array_like
+            The 2D image array.
+
+        Returns
+        -------
+        table : `~astropy.table.Table`
+            A table of found objects with the following parameters:
+
+            * ``id``: unique object identification number.
+            * ``xcentroid, ycentroid``: object centroid.
+            * ``sharpness``: object sharpness.
+            * ``roundness1``: object roundness based on symmetry.
+            * ``roundness2``: object roundness based on marginal Gaussian
+              fits.
+            * ``npix``: number of pixels in the Gaussian kernel.
+            * ``sky``: the input ``sky`` parameter.
+            * ``peak``: the peak, sky-subtracted, pixel value of the object.
+            * ``flux``: the object flux calculated as the peak density in
+              the convolved image divided by the detection threshold.  This
+              derivation matches that of `DAOFIND`_ if ``sky`` is 0.0.
+            * ``mag``: the object instrumental magnitude calculated as
+              ``-2.5 * log10(flux)``.  The derivation matches that of
+              `DAOFIND`_ if ``sky`` is 0.0.
+
+        Notes
+        -----
+        For the convolution step, this routine sets pixels beyond the
+        image borders to 0.0.  The equivalent parameters in IRAF's
+        `starfind`_ are ``boundary='constant'`` and ``constant=0.0``.
+
+        IRAF's `starfind`_ uses ``hwhmpsf``, ``fradius``, and ``sepmin``
+        as input parameters.  The equivalent input values for
+        `~photutils.detection.IRAFStarFinder` are:
+
+        * ``fwhm = hwhmpsf * 2``
+        * ``sigma_radius = fradius * sqrt(2.0*log(2.0))``
+        * ``minsep_fwhm = 0.5 * sepmin``
+
+        The main differences between
+        `~photutils.detection.DAOStarFinder` and
+        `~photutils.detection.IRAFStarFinder` are:
+
+        * `~photutils.detection.IRAFStarFinder` always uses a 2D
+          circular Gaussian kernel, while
+          `~photutils.detection.DAOStarFinder` can use an elliptical
+          Gaussian kernel.
+
+        * `~photutils.detection.IRAFStarFinder` calculates the objects'
+          centroid, roundness, and sharpness using image moments.
+
+        .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+        .. _starfind: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+        """
+
+        raise NotImplementedError
+
+
+class DAOStarFinder(StarFinderBase):
+    """
+    Detect stars in an image using the DAOFIND (`Stetson 1987
+    <http://adsabs.harvard.edu/abs/1987PASP...99..191S>`_) algorithm.
+
+    DAOFIND (`Stetson 1987; PASP 99, 191
     <http://adsabs.harvard.edu/abs/1987PASP...99..191S>`_) searches
     images for local density maxima that have a peak amplitude greater
     than ``threshold`` (approximately; ``threshold`` is applied to a
@@ -30,13 +118,11 @@ def daofind(data, threshold, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5,
     Gaussian kernel.  The Gaussian kernel is defined by the ``fwhm``,
     ``ratio``, ``theta``, and ``sigma_radius`` input parameters.
 
-    .. _DAOFIND: http://iraf.net/irafhelp.php?val=daofind&help=Help+Page
-
-    ``daofind`` finds the object centroid by fitting the the marginal x
+    ``DAOStarFinder`` finds the object centroid by fitting the marginal x
     and y 1D distributions of the Gaussian kernel to the marginal x and
     y distributions of the input (unconvolved) ``data`` image.
 
-    ``daofind`` calculates the object roundness using two methods.  The
+    ``DAOStarFinder`` calculates the object roundness using two methods. The
     ``roundlo`` and ``roundhi`` bounds are applied to both measures of
     roundness.  The first method (``roundness1``; called ``SROUND`` in
     `DAOFIND`_) is based on the source symmetry and is the ratio of a
@@ -56,80 +142,45 @@ def daofind(data, threshold, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5,
 
     Parameters
     ----------
-    data : array_like
-        The 2D array of the image.
-
     threshold : float
         The absolute image value above which to select sources.
-
     fwhm : float
         The full-width half-maximum (FWHM) of the major axis of the
         Gaussian kernel in units of pixels.
-
     ratio : float, optional
         The ratio of the minor to major axis standard deviations of the
         Gaussian kernel.  ``ratio`` must be strictly positive and less
         than or equal to 1.0.  The default is 1.0 (i.e., a circular
         Gaussian kernel).
-
     theta : float, optional
         The position angle (in degrees) of the major axis of the
         Gaussian kernel measured counter-clockwise from the positive x
         axis.
-
     sigma_radius : float, optional
         The truncation radius of the Gaussian kernel in units of sigma
         (standard deviation) [``1 sigma = FWHM /
         (2.0*sqrt(2.0*log(2.0)))``].
-
     sharplo : float, optional
         The lower bound on sharpness for object detection.
-
     sharphi : float, optional
         The upper bound on sharpness for object detection.
-
     roundlo : float, optional
         The lower bound on roundess for object detection.
-
     roundhi : float, optional
         The upper bound on roundess for object detection.
-
     sky : float, optional
         The background sky level of the image.  Setting ``sky`` affects
         only the output values of the object ``peak``, ``flux``, and
         ``mag`` values.  The default is 0.0, which should be used to
         replicate the results from `DAOFIND`_.
-
     exclude_border : bool, optional
         Set to `True` to exclude sources found within half the size of
         the convolution kernel from the image borders.  The default is
         `False`, which is the mode used by `DAOFIND`_.
 
-    Returns
-    -------
-    table : `~astropy.table.Table`
-
-        A table of found objects with the following parameters:
-
-        * ``id``: unique object identification number.
-        * ``xcentroid, ycentroid``: object centroid.
-        * ``sharpness``: object sharpness.
-        * ``roundness1``: object roundness based on symmetry.
-        * ``roundness2``: object roundness based on marginal Gaussian
-          fits.
-        * ``npix``: number of pixels in the Gaussian kernel.
-        * ``sky``: the input ``sky`` parameter.
-        * ``peak``: the peak, sky-subtracted, pixel value of the object.
-        * ``flux``: the object flux calculated as the peak density in
-          the convolved image divided by the detection threshold.  This
-          derivation matches that of `DAOFIND`_ if ``sky`` is 0.0.
-        * ``mag``: the object instrumental magnitude calculated as
-          ``-2.5 * log10(flux)``.  The derivation matches that of
-          `DAOFIND`_ if ``sky`` is 0.0.
-
     See Also
     --------
-    irafstarfind
+    IRAFStarFinder
 
     Notes
     -----
@@ -139,38 +190,54 @@ def daofind(data, threshold, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5,
 
     References
     ----------
-
     .. [1] Stetson, P. 1987; PASP 99, 191 (http://adsabs.harvard.edu/abs/1987PASP...99..191S)
-    .. [2] http://iraf.net/irafhelp.php?val=daofind&help=Help+Page
-    .. [3] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+    .. [2] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+
+    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
     """
 
-    daofind_kernel = _FindObjKernel(fwhm, ratio, theta, sigma_radius)
-    threshold *= daofind_kernel.relerr
-    objs = _findobjs(data, threshold, daofind_kernel,
-                     exclude_border=exclude_border)
-    tbl = _daofind_properties(objs, threshold, daofind_kernel, sky)
-    if len(objs) == 0:
-        warnings.warn('No sources were found.', AstropyUserWarning)
-        return tbl     # empty table
-    table_mask = ((tbl['sharpness'] > sharplo) &
-                  (tbl['sharpness'] < sharphi) &
-                  (tbl['roundness1'] > roundlo) &
-                  (tbl['roundness1'] < roundhi) &
-                  (tbl['roundness2'] > roundlo) &
-                  (tbl['roundness2'] < roundhi))
-    tbl = tbl[table_mask]
-    idcol = Column(name='id', data=np.arange(len(tbl)) + 1)
-    tbl.add_column(idcol, 0)
-    if len(tbl) == 0:
-        warnings.warn('Sources were found, but none pass the sharpness and '
-                      'roundness criteria.', AstropyUserWarning)
-    return tbl
+    def __init__(self, threshold, fwhm, ratio=1.0, theta=0.0,
+                 sigma_radius=1.5, sharplo=0.2, sharphi=1.0, roundlo=-1.0,
+                 roundhi=1.0, sky=0.0, exclude_border=False):
+        self.threshold = threshold
+        self.fwhm = fwhm
+        self.ratio = ratio
+        self.theta = theta
+        self.sigma_radius = sigma_radius
+        self.sharplo = sharplo
+        self.sharphi = sharphi
+        self.roundlo = roundlo
+        self.roundhi = roundhi
+        self.sky = sky
+        self.exclude_border = exclude_border
+
+    def find_stars(self, data):
+        daofind_kernel = _FindObjKernel(self.fwhm, self.ratio, self.theta,
+                                        self.sigma_radius)
+        self.threshold *= daofind_kernel.relerr
+        objs = _findobjs(data, self.threshold, daofind_kernel,
+                         exclude_border=self.exclude_border)
+        tbl = _daofind_properties(objs, self.threshold, daofind_kernel,
+                                  self.sky)
+        if len(objs) == 0:
+            warnings.warn('No sources were found.', AstropyUserWarning)
+            return tbl     # empty table
+        table_mask = ((tbl['sharpness'] > self.sharplo) &
+                      (tbl['sharpness'] < self.sharphi) &
+                      (tbl['roundness1'] > self.roundlo) &
+                      (tbl['roundness1'] < self.roundhi) &
+                      (tbl['roundness2'] > self.roundlo) &
+                      (tbl['roundness2'] < self.roundhi))
+        tbl = tbl[table_mask]
+        idcol = Column(name='id', data=np.arange(len(tbl)) + 1)
+        tbl.add_column(idcol, 0)
+        if len(tbl) == 0:
+            warnings.warn('Sources were found, but none pass the sharpness '
+                          'and roundness criteria.', AstropyUserWarning)
+        return tbl
 
 
-def irafstarfind(data, threshold, fwhm, sigma_radius=1.5, minsep_fwhm=2.5,
-                 sharplo=0.5, sharphi=2.0, roundlo=0.0, roundhi=0.2,
-                 sky=None, exclude_border=False):
+class IRAFStarFinder(StarFinderBase):
     """
     Detect stars in an image using IRAF's "starfind" algorithm.
 
@@ -180,132 +247,86 @@ def irafstarfind(data, threshold, fwhm, sigma_radius=1.5, minsep_fwhm=2.5,
     ``fwhm``.  The objects' centroid, roundness (ellipticity), and
     sharpness are calculated using image moments.
 
-    .. _starfind: http://iraf.net/irafhelp.php?val=starfind&help=Help+Page
-
     Parameters
     ----------
-    data : array_like
-        The 2D array of the image.
-
     threshold : float
         The absolute image value above which to select sources.
-
     fwhm : float
         The full-width half-maximum (FWHM) of the 2D circular Gaussian
         kernel in units of pixels.
-
     minsep_fwhm : float, optional
         The minimum separation for detected objects in units of
         ``fwhm``.
-
     sigma_radius : float, optional
         The truncation radius of the Gaussian kernel in units of sigma
         (standard deviation) [``1 sigma = FWHM /
         2.0*sqrt(2.0*log(2.0))``].
-
     sharplo : float, optional
         The lower bound on sharpness for object detection.
-
     sharphi : float, optional
         The upper bound on sharpness for object detection.
-
     roundlo : float, optional
         The lower bound on roundess for object detection.
-
     roundhi : float, optional
         The upper bound on roundess for object detection.
-
     sky : float, optional
         The background sky level of the image.  Inputing a ``sky`` value
         will override the background sky estimate.  Setting ``sky``
         affects only the output values of the object ``peak``, ``flux``,
         and ``mag`` values.  The default is ``None``, which means the
         sky value will be estimated using the `starfind`_ method.
-
     exclude_border : bool, optional
         Set to `True` to exclude sources found within half the size of
         the convolution kernel from the image borders.  The default is
         `False`, which is the mode used by `starfind`_.
 
-    Returns
-    -------
-    table : `~astropy.table.Table`
-
-        A table of found objects with the following parameters:
-
-        * ``id``: unique object identification number.
-        * ``xcentroid, ycentroid``: object centroid (zero-based origin).
-        * ``fwhm``: estimate of object FWHM from image moments.
-        * ``sharpness``: object sharpness calculated from image moments.
-        * ``roundness``: object ellipticity calculated from image moments.
-        * ``pa``:  object position angle in degrees from the positive x
-          axis calculated from image moments.
-        * ``npix``: number of pixels in the object used to calculate
-          ``flux``.
-        * ``sky``: the derived background sky value, unless ``sky`` was
-          input.  If ``sky`` was input, then that value overrides the
-          background sky estimation.
-        * ``peak``: the peak, sky-subtracted, pixel value of the object.
-        * ``flux``: the object sky-subtracted flux, calculated by
-          summing object pixels over the Gaussian kernel.  The
-          derivation matches that of `starfind`_ if ``sky`` is ``None``.
-        * ``mag``: the object instrumental magnitude calculated as
-          ``-2.5 * log10(flux)``.  The derivation matches that of
-          `starfind`_ if ``sky`` is ``None``.
-
     See Also
     --------
-    daofind
-
-    Notes
-    -----
-    For the convolution step, this routine sets pixels beyond the image
-    borders to 0.0.  The equivalent parameters in `starfind`_ are
-    ``boundary='constant'`` and ``constant=0.0``.
-
-    IRAF's `starfind`_ uses ``hwhmpsf``, ``fradius``, and ``sepmin`` as
-    input parameters.  The equivalent input values for ``irafstarfind``
-    are:
-
-    * ``fwhm = hwhmpsf * 2``
-    * ``sigma_radius = fradius * sqrt(2.0*log(2.0))``
-    * ``minsep_fwhm = 0.5 * sepmin``
-
-    The main differences between ``daofind`` and ``irafstarfind`` are:
-
-    * ``irafstarfind`` always uses a 2D circular Gaussian kernel,
-      while ``daofind`` can use an elliptical Gaussian kernel.
-
-    * ``irafstarfind`` calculates the objects' centroid, roundness,
-      and sharpness using image moments.
+    DAOStarFinder
 
     References
     ----------
-    .. [1] http://iraf.net/irafhelp.php?val=starfind&help=Help+Page
-    .. [2] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+    .. [1] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+
+    .. _starfind: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
     """
 
-    starfind_kernel = _FindObjKernel(fwhm, ratio=1.0, theta=0.0,
-                                     sigma_radius=sigma_radius)
-    min_separation = max(2, int((fwhm * minsep_fwhm) + 0.5))
-    objs = _findobjs(data, threshold, starfind_kernel,
-                     min_separation=min_separation,
-                     exclude_border=exclude_border)
-    tbl = _irafstarfind_properties(objs, starfind_kernel, sky)
-    if len(objs) == 0:
-        warnings.warn('No sources were found.', AstropyUserWarning)
-        return tbl     # empty table
-    table_mask = ((tbl['sharpness'] > sharplo) &
-                  (tbl['sharpness'] < sharphi) &
-                  (tbl['roundness'] > roundlo) &
-                  (tbl['roundness'] < roundhi))
-    tbl = tbl[table_mask]
-    idcol = Column(name='id', data=np.arange(len(tbl)) + 1)
-    tbl.add_column(idcol, 0)
-    if len(tbl) == 0:
-        warnings.warn('Sources were found, but none pass the sharpness and '
-                      'roundness criteria.', AstropyUserWarning)
-    return tbl
+    def __init__(self, threshold, fwhm, sigma_radius=1.5, minsep_fwhm=2.5,
+                 sharplo=0.5, sharphi=2.0, roundlo=0.0, roundhi=0.2, sky=None,
+                 exclude_border=False):
+        self.threshold = threshold
+        self.fwhm = fwhm
+        self.sigma_radius = sigma_radius
+        self.minsep_fwhm = minsep_fwhm
+        self.sharplo = sharplo
+        self.sharphi = sharphi
+        self.roundlo = roundlo
+        self.roundhi = roundhi
+        self.sky = sky
+        self.exclude_border = exclude_border
+
+    def find_stars(self, data):
+        starfind_kernel = _FindObjKernel(self.fwhm, ratio=1.0, theta=0.0,
+                                         sigma_radius=self.sigma_radius)
+        min_separation = max(2, int((self.fwhm * self.minsep_fwhm) + 0.5))
+        objs = _findobjs(data, self.threshold, starfind_kernel,
+                         min_separation=min_separation,
+                         exclude_border=self.exclude_border)
+        tbl = _irafstarfind_properties(objs, starfind_kernel, self.sky)
+        if len(objs) == 0:
+            warnings.warn('No sources were found.', AstropyUserWarning)
+            return tbl     # empty table
+        table_mask = ((tbl['sharpness'] > self.sharplo) &
+                      (tbl['sharpness'] < self.sharphi) &
+                      (tbl['roundness'] > self.roundlo) &
+                      (tbl['roundness'] < self.roundhi))
+        tbl = tbl[table_mask]
+        idcol = Column(name='id', data=np.arange(len(tbl)) + 1)
+        tbl.add_column(idcol, 0)
+        if len(tbl) == 0:
+            warnings.warn('Sources were found, but none pass the sharpness '
+                          'and roundness criteria.', AstropyUserWarning)
+        return tbl
 
 
 def _findobjs(data, threshold, kernel, min_separation=None,
@@ -344,6 +365,10 @@ def _findobjs(data, threshold, kernel, min_separation=None,
     objects : list of `_ImgCutout`
         A list of `_ImgCutout` objects containing the image cutout for
         each source.
+
+
+    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+    .. _starfind: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
     """
 
     from scipy import ndimage
@@ -360,8 +385,8 @@ def _findobjs(data, threshold, kernel, min_separation=None,
                     x_kernradius:x_kernradius + data.shape[1]] = data
         data = data_padded
 
-    convolved_data = _convolve_data(data, kernel.kern, mode='constant',
-                                    fill_value=0.0, check_normalization=False)
+    convolved_data = filter_data(data, kernel.kern, mode='constant',
+                                 fill_value=0.0, check_normalization=False)
 
     if not exclude_border:
         # keep border=0 in convolved data
@@ -553,6 +578,8 @@ def _daofind_properties(imgcutouts, threshold, kernel, sky=0.0):
     -------
     table : `~astropy.table.Table`
         A table of the object parameters.
+
+    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
     """
 
     result = defaultdict(list)
@@ -636,6 +663,8 @@ def _daofind_centroid_roundness(obj, kernel):
 
     g_roundness : float
         `DAOFIND`_ roundness (GROUND) statistic.
+
+    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
     """
 
     dx, hx = _daofind_centroidfit(obj, kernel, axis=0)
@@ -766,8 +795,8 @@ class _ImgCutout(object):
 class _FindObjKernel(object):
     """
     Calculate a 2D Gaussian density enhancement kernel.  This kernel has
-    negative wings and sums to zero.  It is used by both `daofind` and
-    `irafstarfind`.
+    negative wings and sums to zero.  It is used by both `DAOStarFinder`
+    and `IRAFStarFinder`.
 
     Parameters
     ----------
@@ -809,10 +838,15 @@ class _FindObjKernel(object):
     """
 
     def __init__(self, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5):
-        assert fwhm > 0, 'FWHM must be positive'
-        assert ((ratio > 0) & (ratio <= 1)), \
-            'ratio must be positive and less than 1'
-        assert sigma_radius > 0, 'sigma_radius must be positive'
+        if fwhm < 0:
+            raise ValueError('fwhm must be positive, '
+                             'got fwhm={0}'.format(fwhm))
+        if ratio <= 0 or ratio > 1:
+            raise ValueError('ratio must be positive and less or equal '
+                             'than 1, got ratio={0}'.format(ratio))
+        if sigma_radius <= 0:
+            raise ValueError('sigma_radius must be positive, got '
+                             'sigma_radius={0}'.format(sigma_radius))
         self.fwhm = fwhm
         self.sigma_radius = sigma_radius
         self.ratio = ratio
