@@ -3,9 +3,9 @@
 Geometry subpackage for low-level geometry functions.
 """
 
-from .circular_overlap import *
-from .elliptical_overlap import *
-from .rectangular_overlap import *
+from .circular_overlap import *       # noqa
+from .elliptical_overlap import *     # noqa
+from .rectangular_overlap import *    # noqa
 
 
 __all__ = ['circular_overlap_grid', 'elliptical_overlap_grid',
