@@ -4,12 +4,12 @@ from __future__ import (absolute_import, division, print_function,
 
 import numpy as np
 from numpy.testing import assert_allclose
-from astropy.tests.helper import pytest
+import pytest
 
 from ..core import data_properties
 
 try:
-    import skimage
+    import skimage    # noqa
     HAS_SKIMAGE = True
 except ImportError:
     HAS_SKIMAGE = False
