@@ -5,31 +5,18 @@ Models for doing PSF/PRF fitting photometry on image data.
 
 from __future__ import division
 import warnings
-import numpy as np
 import copy
-from astropy.table import Table
+
+import numpy as np
 from astropy.modeling import models, Parameter, Fittable2DModel
-from astropy.modeling.fitting import LevMarLSQFitter
-from astropy.nddata.utils import subpixel_indices
 from astropy.utils.exceptions import AstropyWarning
-from ..utils import mask_to_mirrored_num
-from ..extern.nddata_compat import extract_array
+
 
 __all__ = ['FittableImageModel', 'NonNormalizable',
            'IntegratedGaussianPRF', 'PRFAdapter',
            'prepare_psf_model', 'get_grouped_psf_model']
 
 
-try:
-    import scipy
-    HAS_SCIPY = True
-    from distutils.version import LooseVersion
-    SCIPY_VER_GE_014 = (LooseVersion(scipy.version.full_version) >=
-                        LooseVersion('0.14'))
-except ImportError:
-    HAS_SCIPY = False
-
-
 class NonNormalizable(AstropyWarning):
     """
     Used to indicate that a :py:class:`FittableImageModel` model is
@@ -85,7 +72,7 @@ class FittableImageModel(Fittable2DModel):
         computed so that
 
         .. math::
-            N \cdot C \cdot \Sigma_{i,j}D_{i,j} = 1,
+            N \\cdot C \\cdot \\Sigma_{i,j}D_{i,j} = 1,
 
         where *N* is the normalization constant, *C* is correction factor
         given by the parameter ``normalization_correction``, and
@@ -184,27 +171,29 @@ class FittableImageModel(Fittable2DModel):
 
     def _compute_normalization(self, normalize):
         """
-        Helper function that computes (corrected) normalization factor of the
-        original image data. This quantity is computed as the
-        inverse "raw image norm" (or total "flux" of model's image) corrected
-        by the ``normalization_correction``:
+        Helper function that computes (corrected) normalization factor
+        of the original image data. This quantity is computed as the
+        inverse "raw image norm" (or total "flux" of model's image)
+        corrected by the ``normalization_correction``:
 
         .. math::
-            N = 1/(\Phi * C),
+            N = 1/(\\Phi * C),
 
-        where :math:`\Phi` is the "total flux" of model's image as computed
-        by `_compute_raw_image_norm` and *C* is the normalization correction
-        factor. :math:`\Phi` is computed only once if it has not been
-        previously computed. Otherwise, the existing (stored) value of
-        :math:`\Phi` is not modified as :py:class:`FittableImageModel`
-        does not allow image data to be modified after the object is created.
+        where :math:`\\Phi` is the "total flux" of model's image as
+        computed by `_compute_raw_image_norm` and *C* is the
+        normalization correction factor. :math:`\\Phi` is computed only
+        once if it has not been previously computed. Otherwise, the
+        existing (stored) value of :math:`\\Phi` is not modified as
+        :py:class:`FittableImageModel` does not allow image data to be
+        modified after the object is created.
 
         .. note::
-            Normally, this function should not be called by the end-user. It
-            is intended to be overriden in a subclass if one desires to change
-            the way the normalization factor is computed.
-
+            Normally, this function should not be called by the
+            end-user. It is intended to be overriden in a subclass if
+            one desires to change the way the normalization factor is
+            computed.
         """
+
         self._normalization_constant = 1.0 / self._normalization_correction
 
         if normalize:
@@ -243,7 +232,7 @@ class FittableImageModel(Fittable2DModel):
         """
         try:
             value = float(value)
-        except:
+        except ValueError:
             raise ValueError('Oversampling factor must be a scalar')
         if value <= 0:
             raise ValueError('Oversampling factor must be greater than 0')
@@ -462,23 +451,7 @@ class FittableImageModel(Fittable2DModel):
         yi = self._oversampling * (np.asarray(y) - y_0) + self._y_origin
 
         f = flux * self._normalization_constant
-
-        if SCIPY_VER_GE_014:
-            evaluated_model = f * self.interpolator.ev(xi, yi)
-
-        else:
-            # Flatten x and y arguments in order to evaluate in SCIPY versions
-            # earlier than 0.14.0. This essentially replicates the code
-            # in 'RectBivariateSpline.ev()' method in versions >= 0.14.0.
-            if xi.shape != yi.shape:
-                xi, yi = np.broadcast_arrays(xi, yi)
-            xi_flat = xi.ravel()
-            yi_flat = yi.ravel()
-
-            evaluated_model = f * self.interpolator.ev(xi_flat, yi_flat)
-
-            # reshape evaluated_model to the original shape of x & y arguments:
-            evaluated_model = evaluated_model.reshape(xi.shape)
+        evaluated_model = f * self.interpolator.ev(xi, yi)
 
         if self._fill_value is not None:
             # find indices of pixels that are outside the input pixel grid and
@@ -773,7 +746,7 @@ def prepare_psf_model(psfmodel, xname=None, yname=None, fluxname=None,
     return outmod
 
 
-def get_grouped_psf_model(template_psf_model, star_group):
+def get_grouped_psf_model(template_psf_model, star_group, pars_to_set):
     """
     Construct a joint PSF model which consists of a sum of PSF's templated on
     a specific model, but whose parameters are given by a table of objects.
@@ -795,11 +768,11 @@ def get_grouped_psf_model(template_psf_model, star_group):
     """
 
     group_psf = None
-    for i in range(len(star_group)):
+
+    for star in star_group:
         psf_to_add = template_psf_model.copy()
-        psf_to_add.flux = star_group['flux_0'][i]
-        psf_to_add.x_0 = star_group['x_0'][i]
-        psf_to_add.y_0 = star_group['y_0'][i]
+        for param_tab_name, param_name in pars_to_set.items():
+            setattr(psf_to_add, param_name, star[param_tab_name])
 
         if group_psf is None:
             # this is the first one only
