@@ -5,25 +5,20 @@ from __future__ import division
 import numpy as np
 import warnings
 
-from astropy.table import Table, vstack, hstack
 from astropy.modeling.fitting import LevMarLSQFitter
-from astropy.stats import gaussian_sigma_to_fwhm
+from astropy.nddata.utils import overlap_slices
+from astropy.stats import gaussian_sigma_to_fwhm, SigmaClip
+from astropy.table import Table, Column, vstack, hstack
+from astropy.utils import deprecated_renamed_argument
 from astropy.utils.exceptions import AstropyUserWarning
-from .funcs import subtract_psf
+
+from . import DAOGroup
+from .funcs import subtract_psf, _extract_psf_fitting_names
 from .models import get_grouped_psf_model
 from ..aperture import CircularAperture, aperture_photometry
-
-from ..background import MMMBackground, SigmaClip
+from ..background import MMMBackground
 from ..detection import DAOStarFinder
-from . import DAOGroup
 
-from astropy.utils import minversion
-ASTROPY_LT_1_1 = not minversion('astropy', '1.1')
-
-if ASTROPY_LT_1_1:
-    from ..extern.nddata_compat import _overlap_slices_astropy1p1 as overlap_slices
-else:
-    from astropy.nddata.utils import overlap_slices
 
 __all__ = ['BasicPSFPhotometry', 'IterativelySubtractedPSFPhotometry',
            'DAOPhotPSFPhotometry']
@@ -31,15 +26,16 @@ __all__ = ['BasicPSFPhotometry', 'IterativelySubtractedPSFPhotometry',
 
 class BasicPSFPhotometry(object):
     """
-    This class implements a PSF photometry algorithm that can find sources in an
-    image, group overlapping sources into a single model, fit the model to the
-    sources, and subtracting the models from the image. This is roughly
-    equivalent to the DAOPHOT routines FIND, GROUP, NSTAR, and SUBTRACT.
-    This implementation allows a flexible and customizable interface to
-    perform photometry. For instance, one is able to use different
-    implementations for grouping and finding sources by using ``group_maker``
-    and ``finder`` respectivelly. In addition, sky background estimation is
-    performed by ``bkg_estimator``.
+    This class implements a PSF photometry algorithm that can find
+    sources in an image, group overlapping sources into a single model,
+    fit the model to the sources, and subtracting the models from the
+    image. This is roughly equivalent to the DAOPHOT routines FIND,
+    GROUP, NSTAR, and SUBTRACT.  This implementation allows a flexible
+    and customizable interface to perform photometry. For instance, one
+    is able to use different implementations for grouping and finding
+    sources by using ``group_maker`` and ``finder`` respectivelly. In
+    addition, sky background estimation is performed by
+    ``bkg_estimator``.
 
     Parameters
     ----------
@@ -99,11 +95,11 @@ class BasicPSFPhotometry(object):
 
     Notes
     -----
-    Note that an ambiguity arises whenever ``finder`` and ``positions``
-    (keyword argument for ``do_photometry`)` are both not ``None``. In this
-    case, ``finder`` is ignored and initial guesses are taken from
-    ``positions``. In addition, an warning is raised to remaind the user about
-    this behavior.
+    Note that an ambiguity arises whenever ``finder`` and
+    ``init_guesses`` (keyword argument for ``do_photometry``) are both
+    not ``None``. In this case, ``finder`` is ignored and initial
+    guesses are taken from ``init_guesses``. In addition, an warning is
+    raised to remaind the user about this behavior.
 
     If there are problems with fitting large groups, change the
     parameters of the grouping algorithm to reduce the number of sources
@@ -127,6 +123,8 @@ class BasicPSFPhotometry(object):
         self.fitshape = fitshape
         self.finder = finder
         self.aperture_radius = aperture_radius
+        self._pars_to_set = None
+        self._pars_to_output = None
         self._residual_image = None
 
     @property
@@ -182,16 +180,17 @@ class BasicPSFPhotometry(object):
 
         return self._residual_image
 
-    def __call__(self, image, positions=None):
+    @deprecated_renamed_argument('positions', 'init_guesses', '0.4')
+    def __call__(self, image, init_guesses=None):
         """
         Performs PSF photometry. See `do_photometry` for more details
         including the `__call__` signature.
         """
 
-        return self.do_photometry(image, positions)
+        return self.do_photometry(image, init_guesses)
 
-
-    def do_photometry(self, image, positions=None):
+    @deprecated_renamed_argument('positions', 'init_guesses', '0.4')
+    def do_photometry(self, image, init_guesses=None):
         """
         Perform PSF photometry in ``image``.
 
@@ -200,30 +199,38 @@ class BasicPSFPhotometry(object):
         ``image``. A compound model, in fact a sum of ``psf_model``,
         will be fitted to groups of stars automatically identified by
         ``group_maker``. Also, ``image`` is not assumed to be background
-        subtracted.  If ``positions`` are not ``None`` then this method uses
-        ``positions`` as initial guesses for the centroids. If
-        the centroid positions are set as ``fixed`` in the PSF model
-        ``psf_model``, then the optimizer will only consider the flux as
-        a variable.
+        subtracted.  If ``init_guesses`` are not ``None`` then this
+        method uses ``init_guesses`` as initial guesses for the
+        centroids. If the centroid positions are set as ``fixed`` in the
+        PSF model ``psf_model``, then the optimizer will only consider
+        the flux as a variable.
 
         Parameters
         ----------
         image : 2D array-like, `~astropy.io.fits.ImageHDU`, `~astropy.io.fits.HDUList`
             Image to perform photometry.
-        positions: `~astropy.table.Table`
-            Positions (in pixel coordinates) at which to *start* the fit
-            for each object. Columns 'x_0' and 'y_0' must be present.
-            'flux_0' can also be provided to set initial fluxes.
-            If 'flux_0' is not provided, aperture photometry is used to
-            estimate initial values for the fluxes.
+        init_guesses: `~astropy.table.Table`
+            Table which contains the initial guesses (estimates) for the
+            set of parameters. Columns 'x_0' and 'y_0' which represent
+            the positions (in pixel coordinates) for each object must be
+            present.  'flux_0' can also be provided to set initial
+            fluxes.  If 'flux_0' is not provided, aperture photometry is
+            used to estimate initial values for the fluxes. Additional
+            columns of the form '<parametername>_0' will be used to set
+            the initial guess for any parameters of the ``psf_model``
+            model that are not fixed.
 
         Returns
         -------
         output_tab : `~astropy.table.Table` or None
             Table with the photometry results, i.e., centroids and
             fluxes estimations and the initial estimates used to start
-            the fitting process.
-            None is returned if no sources are found in ``image``.
+            the fitting process. Uncertainties on the fitted parameters
+            are reported as columns called ``<paramname>_unc`` provided
+            that the fitter object contains a dictionary called
+            ``fit_info`` with the key ``param_cov``, which contains the
+            covariance matrix. If ``param_cov`` is not present,
+            uncertanties are not reported.
         """
 
         if self.bkg_estimator is not None:
@@ -236,32 +243,32 @@ class BasicPSFPhotometry(object):
                 self.aperture_radius = (self.psf_model.sigma.value *
                                         gaussian_sigma_to_fwhm)
 
-        if positions is not None:
+        if init_guesses is not None:
             # make sure the code does not modify user's input
-            positions = positions.copy()
-
+            init_guesses = init_guesses.copy()
             if self.aperture_radius is None:
-                if 'flux_0' not in positions.colnames:
-                    raise ValueError('aperture_radius is None and could not be '
-                                     'determined by psf_model. Please, either '
-                                     'provided a value for aperture_radius or '
-                                     'define fwhm/sigma at psf_model.')
+                if 'flux_0' not in init_guesses.colnames:
+                    raise ValueError('aperture_radius is None and could not '
+                                     'be determined by psf_model. Please, '
+                                     'either provided a value for '
+                                     'aperture_radius or define fwhm/sigma '
+                                     'at psf_model.')
 
             if self.finder is not None:
-                warnings.warn('Both positions and finder are different than '
-                              'None, which is ambiguous. finder is going to '
-                              'be ignored.', AstropyUserWarning)
+                warnings.warn('Both init_guesses and finder are different '
+                              'than None, which is ambiguous. finder is '
+                              'going to be ignored.', AstropyUserWarning)
 
-            if 'flux_0' not in positions.colnames:
-                apertures = CircularAperture((positions['x_0'],
-                                              positions['y_0']),
+            if 'flux_0' not in init_guesses.colnames:
+                apertures = CircularAperture((init_guesses['x_0'],
+                                              init_guesses['y_0']),
                                              r=self.aperture_radius)
 
-                positions['flux_0'] = aperture_photometry(
+                init_guesses['flux_0'] = aperture_photometry(
                     image, apertures)['aperture_sum']
         else:
             if self.finder is None:
-                raise ValueError('Finder cannot be None if positions are '
+                raise ValueError('Finder cannot be None if init_guesses are '
                                  'not given.')
             sources = self.finder(image)
             if len(sources) > 0:
@@ -269,18 +276,25 @@ class BasicPSFPhotometry(object):
                                               sources['ycentroid']),
                                              r=self.aperture_radius)
 
-                sources['aperture_flux'] = aperture_photometry(image,
-                        apertures)['aperture_sum']
+                sources['aperture_flux'] = aperture_photometry(
+                    image, apertures)['aperture_sum']
 
-                positions = Table(names=['x_0', 'y_0', 'flux_0'],
-                                  data=[sources['xcentroid'],
-                                  sources['ycentroid'],
-                                  sources['aperture_flux']])
+                init_guesses = Table(names=['x_0', 'y_0', 'flux_0'],
+                                     data=[sources['xcentroid'],
+                                           sources['ycentroid'],
+                                           sources['aperture_flux']])
 
+        self._define_fit_param_names()
+        for p0, param in self._pars_to_set.items():
+            if p0 not in init_guesses.colnames:
+                init_guesses[p0] = (len(init_guesses) *
+                                    [getattr(self.psf_model, param).value])
 
-        star_groups = self.group_maker(positions)
+        star_groups = self.group_maker(init_guesses)
         output_tab, self._residual_image = self.nstar(image, star_groups)
-        output_tab = hstack([positions, output_tab])
+
+        star_groups = star_groups.group_by('group_id')
+        output_tab = hstack([star_groups, output_tab])
 
         return output_tab
 
@@ -299,7 +313,10 @@ class BasicPSFPhotometry(object):
             This table must contain the following columns: ``id``,
             ``group_id``, ``x_0``, ``y_0``, ``flux_0``.  ``x_0`` and
             ``y_0`` are initial estimates of the centroids and
-            ``flux_0`` is an initial estimate of the flux.
+            ``flux_0`` is an initial estimate of the flux. Additionally,
+            columns named as ``<param_name>_0`` are required if any
+            other parameter in the psf model is free (i.e., the
+            ``fixed`` attribute of that parameter is ``False``).
 
         Returns
         -------
@@ -309,16 +326,22 @@ class BasicPSFPhotometry(object):
             Residual image.
         """
 
-        result_tab = Table([[], [], [], [], []],
-                           names=('id', 'group_id', 'x_fit', 'y_fit',
-                                  'flux_fit'),
-                           dtype=('i4', 'i4', 'f8', 'f8', 'f8'))
-        star_groups = star_groups.group_by('group_id')
+        result_tab = Table()
+        for param_tab_name in self._pars_to_output.keys():
+            result_tab.add_column(Column(name=param_tab_name))
+
+        unc_tab = Table()
+        for param, isfixed in self.psf_model.fixed.items():
+            if not isfixed:
+                unc_tab.add_column(Column(name=param + "_unc"))
 
         y, x = np.indices(image.shape)
 
+        star_groups = star_groups.group_by('group_id')
         for n in range(len(star_groups.groups)):
-            group_psf = get_grouped_psf_model(self.psf_model, star_groups.groups[n])
+            group_psf = get_grouped_psf_model(self.psf_model,
+                                              star_groups.groups[n],
+                                              self._pars_to_set)
             usepixel = np.zeros_like(image, dtype=np.bool)
 
             for row in star_groups.groups[n]:
@@ -330,9 +353,13 @@ class BasicPSFPhotometry(object):
             fit_model = self.fitter(group_psf, x[usepixel], y[usepixel],
                                     image[usepixel])
             param_table = self._model_params2table(fit_model,
-                                                   star_groups.groups[n])
+                                                   len(star_groups.groups[n]))
             result_tab = vstack([result_tab, param_table])
 
+            if 'param_cov' in self.fitter.fit_info.keys():
+                unc_tab = vstack([unc_tab,
+                                  self._get_uncertainties(
+                                      len(star_groups.groups[n]))])
             try:
                 from astropy.nddata.utils import NoOverlapError
             except ImportError:
@@ -345,9 +372,72 @@ class BasicPSFPhotometry(object):
             except NoOverlapError:
                 pass
 
+        if 'param_cov' in self.fitter.fit_info.keys():
+            result_tab = hstack([result_tab, unc_tab])
+
         return result_tab, image
 
-    def _model_params2table(self, fit_model, star_group):
+    def _define_fit_param_names(self):
+        """
+        Convenience function to define mappings between the names of the
+        columns in the initial guess table (and the name of the fitted
+        parameters) and the actual name of the parameters in the model.
+
+        This method sets the following parameters on the ``self`` object:
+        * ``pars_to_set`` : Dict which maps the names of the parameters
+          initial guesses to the actual name of the parameter in the
+          model.
+        * ``pars_to_output`` : Dict which maps the names of the fitted
+          parameters to the actual name of the parameter in the model.
+        """
+
+        xname, yname, fluxname = _extract_psf_fitting_names(self.psf_model)
+        self._pars_to_set = {'x_0': xname, 'y_0': yname, 'flux_0': fluxname}
+        self._pars_to_output = {'x_fit': xname, 'y_fit': yname,
+                                'flux_fit': fluxname}
+
+        for p, isfixed in self.psf_model.fixed.items():
+            p0 = p + '_0'
+            pfit = p + '_fit'
+            if p not in (xname, yname, fluxname) and not isfixed:
+                self._pars_to_set[p0] = p
+                self._pars_to_output[pfit] = p
+
+    def _get_uncertainties(self, star_group_size):
+        """
+        Retrieve uncertainties on fitted parameters from the fitter
+        object.
+
+        Parameters
+        ----------
+        star_group_size : int
+            Number of stars in the given group.
+
+        Returns
+        -------
+        unc_tab : `~astropy.table.Table`
+            Table which contains uncertainties on the fitted parameters.
+            The uncertainties are reported as one standard deviation.
+        """
+
+        unc_tab = Table()
+        for param_name in self.psf_model.param_names:
+            if not self.psf_model.fixed[param_name]:
+                unc_tab.add_column(Column(name=param_name + "_unc",
+                                          data=np.empty(star_group_size)))
+
+        if 'param_cov' in self.fitter.fit_info.keys():
+            if self.fitter.fit_info['param_cov'] is not None:
+                k = 0
+                n_fit_params = len(unc_tab.colnames)
+                for i in range(star_group_size):
+                    unc_tab[i] = np.sqrt(np.diag(
+                                          self.fitter.fit_info['param_cov'])
+                                         )[k: k + n_fit_params]
+                    k = k + n_fit_params
+        return unc_tab
+
+    def _model_params2table(self, fit_model, star_group_size):
         """
         Place fitted parameters into an astropy table.
 
@@ -358,33 +448,30 @@ class BasicPSFPhotometry(object):
             in this package like `~photutils.psf.sandbox.DiscretePRF`,
             `~photutils.psf.IntegratedGaussianPRF`, or any other
             suitable 2D model.
-        star_group : ~astropy.table.Table
+        star_group_size : int
+            Number of stars in the given group.
 
         Returns
         -------
-        param_tab : ~astropy.table.Table
+        param_tab : `~astropy.table.Table`
             Table that contains the fitted parameters.
         """
 
-        param_tab = Table([[], [], [], [], []],
-                          names=('id', 'group_id', 'x_fit', 'y_fit',
-                                 'flux_fit'),
-                          dtype=('i4', 'i4', 'f8', 'f8', 'f8'))
+        param_tab = Table()
 
-        if hasattr(fit_model, 'submodel_names'):
-            for i in range(len(fit_model.submodel_names)):
-                param_tab.add_row([[star_group['id'][i]],
-                                   [star_group['group_id'][i]],
-                                   [getattr(fit_model, 'x_0_'+str(i)).value],
-                                   [getattr(fit_model, 'y_0_'+str(i)).value],
-                                   [getattr(fit_model,
-                                            'flux_'+str(i)).value]])
+        for param_tab_name in self._pars_to_output.keys():
+            param_tab.add_column(Column(name=param_tab_name,
+                                        data=np.empty(star_group_size)))
+
+        if star_group_size > 1:
+            for i in range(star_group_size):
+                for param_tab_name, param_name in self._pars_to_output.items():
+                    param_tab[param_tab_name][i] = getattr(fit_model,
+                                                           param_name +
+                                                           '_' + str(i)).value
         else:
-            param_tab.add_row([[star_group['id'][0]],
-                               [star_group['group_id'][0]],
-                               [getattr(fit_model, 'x_0').value],
-                               [getattr(fit_model, 'y_0').value],
-                               [getattr(fit_model, 'flux').value]])
+            for param_tab_name, param_name in self._pars_to_output.items():
+                param_tab[param_tab_name] = getattr(fit_model, param_name).value
 
         return param_tab
 
@@ -393,9 +480,9 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
     """
     This class implements an iterative algorithm to perform point spread
     function photometry in crowded fields. This consists of applying a
-    loop of find sources, make groups, fit groups, subtract groups, and then
-    repeat until no more stars are detected or a given number of iterations is
-    reached.
+    loop of find sources, make groups, fit groups, subtract groups, and
+    then repeat until no more stars are detected or a given number of
+    iterations is reached.
 
     Parameters
     ----------
@@ -441,7 +528,8 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         ``id`` is an integer-valued column starting from ``1``,
         ``xcentroid`` and ``ycentroid`` are center position estimates of
         the sources and ``flux`` contains flux estimates of the sources.
-        See, e.g., `~photutils.detection.DAOStarFinder` or `~photutils.detection.IRAFStarFinder`.
+        See, e.g., `~photutils.detection.DAOStarFinder` or
+        `~photutils.detection.IRAFStarFinder`.
     fitter : `~astropy.modeling.fitting.Fitter` instance
         Fitter object used to compute the optimized centroid positions
         and/or flux of the identified sources. See
@@ -476,11 +564,11 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
                  finder, fitter=LevMarLSQFitter(), niters=3,
                  aperture_radius=None):
 
-        super(IterativelySubtractedPSFPhotometry, self).__init__(group_maker,
-                bkg_estimator, psf_model, fitshape, finder, fitter, aperture_radius)
+        super(IterativelySubtractedPSFPhotometry, self).__init__(
+            group_maker, bkg_estimator, psf_model, fitshape, finder, fitter,
+            aperture_radius)
         self.niters = niters
 
-
     @property
     def niters(self):
         return self._niters
@@ -495,7 +583,7 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
                     raise ValueError('niters must be positive.')
                 else:
                     self._niters = int(value)
-            except:
+            except ValueError:
                 raise ValueError('niters must be None or an integer or '
                                  'convertable into an integer.')
 
@@ -513,7 +601,8 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         else:
             self._finder = value
 
-    def do_photometry(self, image, positions=None):
+    @deprecated_renamed_argument('positions', 'init_guesses', '0.4')
+    def do_photometry(self, image, init_guesses=None):
         """
         Perform PSF photometry in ``image``.
 
@@ -522,40 +611,49 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         ``image``. A compound model, in fact a sum of ``psf_model``,
         will be fitted to groups of stars automatically identified by
         ``group_maker``. Also, ``image`` is not assumed to be background
-        subtracted.  If ``positions`` are not ``None`` then this method uses
-        ``positions`` as initial guesses for the centroids. If
-        the centroid positions are set as ``fixed`` in the PSF model
-        ``psf_model``, then the optimizer will only consider the flux as
-        a variable.
+        subtracted.  If ``init_guesses`` are not ``None`` then this
+        method uses ``init_guesses`` as initial guesses for the
+        centroids. If the centroid positions are set as ``fixed`` in the
+        PSF model ``psf_model``, then the optimizer will only consider
+        the flux as a variable.
 
         Parameters
         ----------
         image : 2D array-like, `~astropy.io.fits.ImageHDU`, `~astropy.io.fits.HDUList`
             Image to perform photometry.
-        positions: `~astropy.table.Table`
-            Positions (in pixel coordinates) at which to *start* the fit
-            for each object. Columns 'x_0' and 'y_0' must be present.
-            'flux_0' can also be provided to set initial fluxes.
-            If 'flux_0' is not provided, aperture photometry is used to
-            estimate initial values for the fluxes.
+        init_guesses: `~astropy.table.Table`
+            Table which contains the initial guesses (estimates) for the
+            set of parameters. Columns 'x_0' and 'y_0' which represent
+            the positions (in pixel coordinates) for each object must be
+            present.  'flux_0' can also be provided to set initial
+            fluxes.  If 'flux_0' is not provided, aperture photometry is
+            used to estimate initial values for the fluxes. Additional
+            columns of the form '<parametername>_0' will be used to set
+            the initial guess for any parameters of the ``psf_model``
+            model that are not fixed.
 
         Returns
         -------
         output_table : `~astropy.table.Table` or None
             Table with the photometry results, i.e., centroids and
             fluxes estimations and the initial estimates used to start
-            the fitting process.
-            None is returned if no sources are found in ``image``.
+            the fitting process. Uncertainties on the fitted parameters
+            are reported as columns called ``<paramname>_unc`` provided
+            that the fitter object contains a dictionary called
+            ``fit_info`` with the key ``param_cov``, which contains the
+            covariance matrix.
         """
 
-        if positions is not None:
+        if init_guesses is not None:
             table = super(IterativelySubtractedPSFPhotometry,
-                self).do_photometry(image, positions)
-            table['iter_detected'] = np.ones(table['x_fit'].shape, dtype=np.int)
+                          self).do_photometry(image, init_guesses)
+            table['iter_detected'] = np.ones(table['x_fit'].shape,
+                                             dtype=np.int32)
 
             # n_start = 2 because it starts in the second iteration
             # since the first iteration is above
-            output_table = self._do_photometry(n_start=2)
+            output_table = self._do_photometry(init_guesses.colnames,
+                                               n_start=2)
             output_table = vstack([table, output_table])
         else:
             if self.bkg_estimator is not None:
@@ -568,18 +666,23 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
                     self.aperture_radius = (self.psf_model.sigma.value *
                                             gaussian_sigma_to_fwhm)
 
-            output_table = self._do_photometry()
+            output_table = self._do_photometry(['x_0', 'y_0', 'flux_0'])
         return output_table
 
-    def _do_photometry(self, n_start=1):
+    def _do_photometry(self, param_tab, n_start=1):
         """
-        Helper function which performs the iterations of the photometry process.
+        Helper function which performs the iterations of the photometry
+        process.
 
         Parameters
         ----------
+        param_names :  list
+            Names of the columns which represent the initial guesses.
+            For example, ['x_0', 'y_0', 'flux_0'], for intial guesses on
+            the center positions and the flux.
         n_start : int
-            Integer representing the start index of the iteration.
-            It is 1 if positions are None, and 2 otherwise.
+            Integer representing the start index of the iteration.  It
+            is 1 if init_guesses are None, and 2 otherwise.
 
         Returns
         -------
@@ -587,15 +690,16 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
             Table with the photometry results, i.e., centroids and
             fluxes estimations and the initial estimates used to start
             the fitting process.
-            None is returned if no sources are found in ``image``.
         """
 
-        output_table = Table([[], [], [], [], [], [], [], [], []],
-                names=('x_0', 'y_0', 'flux_0', 'id', 'group_id',
-                       'iter_detected', 'x_fit', 'y_fit',
-                       'flux_fit'),
-                dtype=('f8', 'f8', 'f8', 'i4', 'i4', 'i4', 'f8', 'f8',
-                       'f8'))
+        output_table = Table()
+        self._define_fit_param_names()
+
+        for (init_parname, fit_parname) in zip(self._pars_to_set.keys(),
+                                               self._pars_to_output.keys()):
+            output_table.add_column(Column(name=init_parname))
+            output_table.add_column(Column(name=fit_parname))
+
         sources = self.finder(self._residual_image)
 
         n = n_start
@@ -603,23 +707,41 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
               (self.niters is None or n <= self.niters)):
             apertures = CircularAperture((sources['xcentroid'],
                                           sources['ycentroid']),
-                                          r=self.aperture_radius)
-            sources['aperture_flux'] = aperture_photometry(self._residual_image,
-                    apertures)['aperture_sum']
-            init_guess_tab = Table(names=['x_0', 'y_0', 'flux_0'],
-                               data=[sources['xcentroid'],
-                               sources['ycentroid'],
-                               sources['aperture_flux']])
+                                         r=self.aperture_radius)
+            sources['aperture_flux'] = aperture_photometry(
+                self._residual_image, apertures)['aperture_sum']
+
+            init_guess_tab = Table(names=['id', 'x_0', 'y_0', 'flux_0'],
+                                   data=[sources['id'], sources['xcentroid'],
+                                         sources['ycentroid'],
+                                         sources['aperture_flux']])
+
+            for param_tab_name, param_name in self._pars_to_set.items():
+                if param_tab_name not in (['x_0', 'y_0', 'flux_0']):
+                    init_guess_tab.add_column(
+                        Column(name=param_tab_name,
+                               data=(getattr(self.psf_model,
+                                             param_name) *
+                                     np.ones(len(sources)))))
 
             star_groups = self.group_maker(init_guess_tab)
-            table, self._residual_image = super(IterativelySubtractedPSFPhotometry,
-                    self).nstar(self._residual_image, star_groups)
-            table = hstack([init_guess_tab, table])
+            table, self._residual_image = super(
+                IterativelySubtractedPSFPhotometry, self).nstar(
+                    self._residual_image, star_groups)
 
-            table['iter_detected'] = n*np.ones(table['x_fit'].shape, dtype=np.int)
+            star_groups = star_groups.group_by('group_id')
+            table = hstack([star_groups, table])
+
+            table['iter_detected'] = n*np.ones(table['x_fit'].shape,
+                                               dtype=np.int32)
 
             output_table = vstack([output_table, table])
-            sources = self.finder(self._residual_image)
+
+            # do not warn if no sources are found beyond the first iteration
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore', AstropyUserWarning)
+                sources = self.finder(self._residual_image)
+
             n += 1
 
         return output_table
@@ -627,32 +749,34 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
 
 class DAOPhotPSFPhotometry(IterativelySubtractedPSFPhotometry):
     """
-    This class implements  an iterative algorithm based on the
-    DAOPHOT algorithm presented by Stetson (1987) to perform point
-    spread function photometry in crowded fields. This consists of
-    applying a loop of find sources, make groups, fit groups, subtract
-    groups, and then repeat until no more stars are detected or a given
-    number of iterations is reached.
+    This class implements  an iterative algorithm based on the DAOPHOT
+    algorithm presented by Stetson (1987) to perform point spread
+    function photometry in crowded fields. This consists of applying a
+    loop of find sources, make groups, fit groups, subtract groups, and
+    then repeat until no more stars are detected or a given number of
+    iterations is reached.
 
-    Basically, this classes uses `~photutils.psf.IterativelySubtractedPSFPhotometry`,
-    but with grouping, finding, and background estimation routines defined a
-    priori. More precisely, this class uses `~photutils.psf.DAOGroup` for
-    grouping, `~photutils.detection.DAOStarFinder` for finding sources, and
-    `~photutils.background.MMMBackground` for background estimation. Those
-    classes are based on GROUP, FIND, and SKY routines used in DAOPHOT,
-    respectively.
+    Basically, this classes uses
+    `~photutils.psf.IterativelySubtractedPSFPhotometry`, but with
+    grouping, finding, and background estimation routines defined a
+    priori. More precisely, this class uses `~photutils.psf.DAOGroup`
+    for grouping, `~photutils.detection.DAOStarFinder` for finding
+    sources, and `~photutils.background.MMMBackground` for background
+    estimation. Those classes are based on GROUP, FIND, and SKY routines
+    used in DAOPHOT, respectively.
 
-    The parameter ``crit_separation`` is associated with `~photutils.psf.DAOGroup`.
-    ``sigma_clip`` is associated with `~photutils.background.MMMBackground`.
-    ``threshold`` and ``fwhm`` are associated with `~photutils.detection.DAOStarFinder`.
+    The parameter ``crit_separation`` is associated with
+    `~photutils.psf.DAOGroup`.  ``sigma_clip`` is associated with
+    `~photutils.background.MMMBackground`.  ``threshold`` and ``fwhm``
+    are associated with `~photutils.detection.DAOStarFinder`.
     Parameters from ``ratio`` to ``roundhi`` are also associated with
     `~photutils.detection.DAOStarFinder`.
 
     Parameters
     ----------
     crit_separation : float or int
-        Distance, in units of pixels, such that any two stars separated by
-        less than this distance will be placed in the same group.
+        Distance, in units of pixels, such that any two stars separated
+        by less than this distance will be placed in the same group.
     threshold : float
         The absolute image value above which to select sources.
     fwhm : float
@@ -677,7 +801,7 @@ class DAOPhotPSFPhotometry(IterativelySubtractedPSFPhotometry):
         number.
     sigma : float, optional
         Number of standard deviations used to perform sigma clip with a
-        `~photutils.SigmaClip` object.
+        `astropy.stats.SigmaClip` object.
     ratio : float, optional
         The ratio of the minor to major axis standard deviations of the
         Gaussian kernel.  ``ratio`` must be strictly positive and less
@@ -729,9 +853,10 @@ class DAOPhotPSFPhotometry(IterativelySubtractedPSFPhotometry):
         Available at: http://adsabs.harvard.edu/abs/1987PASP...99..191S
     """
 
-    def __init__(self, crit_separation, threshold, fwhm, psf_model,
-                 fitshape, sigma=3., ratio=1.0, theta=0.0, sigma_radius=1.5, sharplo=0.2,
-                 sharphi=1.0, roundlo=-1.0, roundhi=1.0, fitter=LevMarLSQFitter(),
+    def __init__(self, crit_separation, threshold, fwhm, psf_model, fitshape,
+                 sigma=3., ratio=1.0, theta=0.0, sigma_radius=1.5,
+                 sharplo=0.2, sharphi=1.0, roundlo=-1.0, roundhi=1.0,
+                 fitter=LevMarLSQFitter(),
                  niters=3, aperture_radius=None):
 
         self.crit_separation = crit_separation
@@ -748,13 +873,13 @@ class DAOPhotPSFPhotometry(IterativelySubtractedPSFPhotometry):
 
         group_maker = DAOGroup(crit_separation=self.crit_separation)
         bkg_estimator = MMMBackground(sigma_clip=SigmaClip(sigma=self.sigma))
-        finder = DAOStarFinder(threshold=self.threshold, fwhm=self.fwhm, ratio=self.ratio,
-                               theta=self.theta, sigma_radius=self.sigma_radius,
+        finder = DAOStarFinder(threshold=self.threshold, fwhm=self.fwhm,
+                               ratio=self.ratio, theta=self.theta,
+                               sigma_radius=self.sigma_radius,
                                sharplo=self.sharplo, sharphi=self.sharphi,
                                roundlo=self.roundlo, roundhi=self.roundhi)
 
-        super(DAOPhotPSFPhotometry, self).__init__(group_maker=group_maker,
-                                                   bkg_estimator=bkg_estimator,
-                                                   psf_model=psf_model, fitshape=fitshape,
-                                                   finder=finder, fitter=fitter, niters=niters,
-                                                   aperture_radius=aperture_radius)
+        super(DAOPhotPSFPhotometry, self).__init__(
+            group_maker=group_maker, bkg_estimator=bkg_estimator,
+            psf_model=psf_model, fitshape=fitshape, finder=finder,
+            fitter=fitter, niters=niters, aperture_radius=aperture_radius)
