@@ -1,39 +1,31 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import division
 
+import pytest
 import numpy as np
-import astropy
-
 from numpy.testing import assert_allclose, assert_array_equal, assert_equal
 
 from astropy.table import Table
-from astropy.stats import gaussian_sigma_to_fwhm
-from astropy.utils import minversion
+from astropy.stats import gaussian_sigma_to_fwhm, SigmaClip
 from astropy.modeling import Parameter, Fittable2DModel
 from astropy.modeling.fitting import LevMarLSQFitter
-from astropy.modeling.models import Gaussian2D
+from astropy.modeling.models import Gaussian2D, Moffat2D
 from astropy.convolution.utils import discretize_model
-from astropy.tests.helper import pytest, catch_warnings
+from astropy.tests.helper import catch_warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
 from ..groupstars import DAOGroup
-from ..models import IntegratedGaussianPRF
-from ..photometry import DAOPhotPSFPhotometry, IterativelySubtractedPSFPhotometry
-from ..photometry import BasicPSFPhotometry
+from ..models import IntegratedGaussianPRF, prepare_psf_model
+from ..photometry import (DAOPhotPSFPhotometry, BasicPSFPhotometry,
+                          IterativelySubtractedPSFPhotometry)
 from ..sandbox import DiscretePRF
-from ...background import SigmaClip, MedianBackground, StdBackgroundRMS
-from ...background import MedianBackground, MMMBackground, SigmaClip
-from ...background import StdBackgroundRMS
-from ...datasets import make_gaussian_sources
-from ...datasets import make_noise_image
+from ...background import StdBackgroundRMS, MMMBackground
+from ...datasets import make_gaussian_sources_image, make_noise_image
 from ...detection import DAOStarFinder
 
 
-ASTROPY_GT_1_1 = minversion('astropy', '1.1')
-
-
 try:
-    import scipy
+    import scipy    # noqa
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
@@ -48,8 +40,6 @@ def make_psf_photometry_objs(std=1, sigma_psf=1):
     daofind = DAOStarFinder(threshold=5.0*std,
                             fwhm=sigma_psf*gaussian_sigma_to_fwhm)
     daogroup = DAOGroup(1.5*sigma_psf*gaussian_sigma_to_fwhm)
-    sigma_clip = SigmaClip(sigma=3.)
-    median_bkg = MedianBackground(sigma_clip)
     threshold = 5. * std
     fwhm = sigma_psf * gaussian_sigma_to_fwhm
     crit_separation = 1.5 * sigma_psf * gaussian_sigma_to_fwhm
@@ -126,13 +116,13 @@ sources3['group_id'] = [1] * 2
 sources3['iter_detected'] = [1, 2]
 
 
-@pytest.mark.xfail('not HAS_SCIPY or not ASTROPY_GT_1_1')
+@pytest.mark.xfail('not HAS_SCIPY')
 @pytest.mark.parametrize("sigma_psf, sources", [(sigma_psfs[2], sources3)])
 def test_psf_photometry_niters(sigma_psf, sources):
     img_shape = (32, 32)
     # generate image with read-out noise (Gaussian) and
     # background noise (Poisson)
-    image = (make_gaussian_sources(img_shape, sources) +
+    image = (make_gaussian_sources_image(img_shape, sources) +
              make_noise_image(img_shape, type='poisson', mean=6.,
                               random_state=1) +
              make_noise_image(img_shape, type='gaussian', mean=0.,
@@ -142,31 +132,43 @@ def test_psf_photometry_niters(sigma_psf, sources):
     bkgrms = StdBackgroundRMS(sigma_clip)
     std = bkgrms(image)
 
-    iter_phot_obj = make_psf_photometry_objs(std, sigma_psf)[1]
-    iter_phot_obj.niters = None
-    result_tab = iter_phot_obj(image)
-    residual_image = iter_phot_obj.get_residual_image()
+    phot_obj = make_psf_photometry_objs(std, sigma_psf)[1:3]
+    for iter_phot_obj in phot_obj:
+        iter_phot_obj.niters = None
 
-    assert_allclose(result_tab['x_fit'], sources['x_mean'], rtol=1e-1)
-    assert_allclose(result_tab['y_fit'], sources['y_mean'], rtol=1e-1)
-    assert_allclose(result_tab['flux_fit'], sources['flux'], rtol=1e-1)
-    assert_array_equal(result_tab['id'], sources['id'])
-    assert_array_equal(result_tab['group_id'], sources['group_id'])
-    assert_array_equal(result_tab['iter_detected'], sources['iter_detected'])
-    assert_allclose(np.mean(residual_image), 0.0, atol=1e1)
+        result_tab = iter_phot_obj(image)
+        residual_image = iter_phot_obj.get_residual_image()
 
-    # make sure image is note overwritten
-    assert_array_equal(cp_image, image)
+        assert (result_tab['x_0_unc'] < 1.96 * sigma_psf /
+                np.sqrt(sources['flux'])).all()
+        assert (result_tab['y_0_unc'] < 1.96 * sigma_psf /
+                np.sqrt(sources['flux'])).all()
+        assert (result_tab['flux_unc'] < 1.96 *
+                np.sqrt(sources['flux'])).all()
+
+        assert_allclose(result_tab['x_fit'], sources['x_mean'], rtol=1e-1)
+        assert_allclose(result_tab['y_fit'], sources['y_mean'], rtol=1e-1)
+        assert_allclose(result_tab['flux_fit'], sources['flux'], rtol=1e-1)
+        assert_array_equal(result_tab['id'], sources['id'])
+        assert_array_equal(result_tab['group_id'], sources['group_id'])
+        assert_array_equal(result_tab['iter_detected'],
+                           sources['iter_detected'])
+        assert_allclose(np.mean(residual_image), 0.0, atol=1e1)
+
+        # make sure image is note overwritten
+        assert_array_equal(cp_image, image)
 
 
-@pytest.mark.xfail('not HAS_SCIPY or not ASTROPY_GT_1_1')
+@pytest.mark.xfail('not HAS_SCIPY')
 @pytest.mark.parametrize("sigma_psf, sources",
                          [(sigma_psfs[0], sources1),
                           (sigma_psfs[1], sources2),
                           # these ensure that the test *fails* if the model
                           # PSFs are the wrong shape
-                          pytest.mark.xfail((sigma_psfs[0]/1.2, sources1)),
-                          pytest.mark.xfail((sigma_psfs[1]*1.2, sources2))])
+                          pytest.param(sigma_psfs[0]/1.2, sources1,
+                                       marks=pytest.mark.xfail()),
+                          pytest.param(sigma_psfs[1]*1.2, sources2,
+                                       marks=pytest.mark.xfail())])
 def test_psf_photometry_oneiter(sigma_psf, sources):
     """
     Tests in an image with a group of two overlapped stars and an
@@ -176,7 +178,7 @@ def test_psf_photometry_oneiter(sigma_psf, sources):
     img_shape = (32, 32)
     # generate image with read-out noise (Gaussian) and
     # background noise (Poisson)
-    image = (make_gaussian_sources(img_shape, sources) +
+    image = (make_gaussian_sources_image(img_shape, sources) +
              make_noise_image(img_shape, type='poisson', mean=6.,
                               random_state=1) +
              make_noise_image(img_shape, type='gaussian', mean=0.,
@@ -191,6 +193,12 @@ def test_psf_photometry_oneiter(sigma_psf, sources):
     for phot_proc in phot_objs:
         result_tab = phot_proc(image)
         residual_image = phot_proc.get_residual_image()
+        assert (result_tab['x_0_unc'] < 1.96 * sigma_psf /
+                np.sqrt(sources['flux'])).all()
+        assert (result_tab['y_0_unc'] < 1.96 * sigma_psf /
+                np.sqrt(sources['flux'])).all()
+        assert (result_tab['flux_unc'] < 1.96 *
+                np.sqrt(sources['flux'])).all()
         assert_allclose(result_tab['x_fit'], sources['x_mean'], rtol=1e-1)
         assert_allclose(result_tab['y_fit'], sources['y_mean'], rtol=1e-1)
         assert_allclose(result_tab['flux_fit'], sources['flux'], rtol=1e-1)
@@ -204,9 +212,14 @@ def test_psf_photometry_oneiter(sigma_psf, sources):
 
         pos = Table(names=['x_0', 'y_0'], data=[sources['x_mean'],
                                                 sources['y_mean']])
+        cp_pos = pos.copy()
+
         result_tab = phot_proc(image, pos)
         residual_image = phot_proc.get_residual_image()
-
+        assert 'x_0_unc' not in result_tab.colnames
+        assert 'y_0_unc' not in result_tab.colnames
+        assert (result_tab['flux_unc'] < 1.96 *
+                np.sqrt(sources['flux'])).all()
         assert_array_equal(result_tab['x_fit'], sources['x_mean'])
         assert_array_equal(result_tab['y_fit'], sources['y_mean'])
         assert_allclose(result_tab['flux_fit'], sources['flux'], rtol=1e-1)
@@ -214,9 +227,12 @@ def test_psf_photometry_oneiter(sigma_psf, sources):
         assert_array_equal(result_tab['group_id'], sources['group_id'])
         assert_allclose(np.mean(residual_image), 0.0, atol=1e1)
 
-        # make sure image is note overwritten
+        # make sure image is not overwritten
         assert_array_equal(cp_image, image)
 
+        # make sure initial guess table is not modified
+        assert_array_equal(cp_pos, pos)
+
         # resets fixed positions
         phot_proc.psf_model.x_0.fixed = False
         phot_proc.psf_model.y_0.fixed = False
@@ -264,6 +280,7 @@ def test_fitshape_erros():
     with pytest.raises(ValueError):
         basic_phot_obj.fitshape = (3, 3, 3)
 
+
 @pytest.mark.xfail('not HAS_SCIPY')
 def test_aperture_radius_errors():
     basic_phot_obj = make_psf_photometry_objs()[0]
@@ -275,48 +292,58 @@ def test_aperture_radius_errors():
     with pytest.raises(ValueError):
         basic_phot_obj.aperture_radius = -3
 
+
 @pytest.mark.xfail('not HAS_SCIPY')
 def test_finder_erros():
     iter_phot_obj = make_psf_photometry_objs()[1]
+
     with pytest.raises(ValueError):
         iter_phot_obj.finder = None
 
     with pytest.raises(ValueError):
-        iter_phot_obj = IterativelySubtractedPSFPhotometry(finder=None,
-                group_maker=DAOGroup(1), bkg_estimator=MMMBackground(),
-                psf_model=IntegratedGaussianPRF(1), fitshape=(11, 11))
+        iter_phot_obj = IterativelySubtractedPSFPhotometry(
+            finder=None, group_maker=DAOGroup(1),
+            bkg_estimator=MMMBackground(),
+            psf_model=IntegratedGaussianPRF(1), fitshape=(11, 11))
 
-@pytest.mark.xfail('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.xfail('not HAS_SCIPY')
 def test_finder_positions_warning():
     basic_phot_obj = make_psf_photometry_objs(sigma_psf=2)[0]
     positions = Table()
     positions['x_0'] = [12.8, 18.2, 25.3]
     positions['y_0'] = [15.7, 16.5, 25.1]
 
-    image = (make_gaussian_sources((32, 32), sources1) +
+    image = (make_gaussian_sources_image((32, 32), sources1) +
              make_noise_image((32, 32), type='poisson', mean=6.,
                               random_state=1))
 
     with catch_warnings(AstropyUserWarning):
-        result_tab = basic_phot_obj(image=image, positions=positions)
+        result_tab = basic_phot_obj(image=image, init_guesses=positions)
         assert_array_equal(result_tab['x_0'], positions['x_0'])
         assert_array_equal(result_tab['y_0'], positions['y_0'])
         assert_allclose(result_tab['x_fit'], positions['x_0'], rtol=1e-1)
         assert_allclose(result_tab['y_fit'], positions['y_0'], rtol=1e-1)
 
-@pytest.mark.xfail('not HAS_SCIPY or not ASTROPY_GT_1_1')
+    with pytest.raises(ValueError):
+        basic_phot_obj.finder = None
+        result_tab = basic_phot_obj(image=image)
+
+
+@pytest.mark.xfail('not HAS_SCIPY')
 def test_aperture_radius():
     img_shape = (32, 32)
 
     # generate image with read-out noise (Gaussian) and
     # background noise (Poisson)
-    image = (make_gaussian_sources(img_shape, sources1) +
+    image = (make_gaussian_sources_image(img_shape, sources1) +
              make_noise_image(img_shape, type='poisson', mean=6.,
                               random_state=1) +
              make_noise_image(img_shape, type='gaussian', mean=0.,
                               stddev=2., random_state=1))
 
     basic_phot_obj = make_psf_photometry_objs()[0]
+
     # test that aperture radius is properly set whenever the PSF model has
     # a `fwhm` attribute
     class PSFModelWithFWHM(Fittable2DModel):
@@ -333,11 +360,37 @@ def test_aperture_radius():
 
     psf_model = PSFModelWithFWHM()
     basic_phot_obj.psf_model = psf_model
-    result_tab = basic_phot_obj(image)
-
+    basic_phot_obj(image)
     assert_equal(basic_phot_obj.aperture_radius, psf_model.fwhm.value)
 
 
+PARS_TO_SET_0 = {'x_0': 'x_0', 'y_0': 'y_0', 'flux_0': 'flux'}
+PARS_TO_OUTPUT_0 = {'x_fit': 'x_0', 'y_fit': 'y_0', 'flux_fit': 'flux'}
+PARS_TO_SET_1 = PARS_TO_SET_0.copy()
+PARS_TO_SET_1['sigma_0'] = 'sigma'
+PARS_TO_OUTPUT_1 = PARS_TO_OUTPUT_0.copy()
+PARS_TO_OUTPUT_1['sigma_fit'] = 'sigma'
+
+
+@pytest.mark.parametrize("actual_pars_to_set, actual_pars_to_output,"
+                         "is_sigma_fixed", [(PARS_TO_SET_0, PARS_TO_OUTPUT_0,
+                                             True),
+                                            (PARS_TO_SET_1, PARS_TO_OUTPUT_1,
+                                             False)])
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_define_fit_param_names(actual_pars_to_set, actual_pars_to_output,
+                                is_sigma_fixed):
+    psf_model = IntegratedGaussianPRF()
+    psf_model.sigma.fixed = is_sigma_fixed
+
+    basic_phot_obj = make_psf_photometry_objs()[0]
+    basic_phot_obj.psf_model = psf_model
+
+    basic_phot_obj._define_fit_param_names()
+    assert_equal(basic_phot_obj._pars_to_set, actual_pars_to_set)
+    assert_equal(basic_phot_obj._pars_to_output, actual_pars_to_output)
+
+
 # tests previously written to psf_photometry
 
 PSF_SIZE = 11
@@ -378,7 +431,8 @@ for x, y, flux in WIDE_INTAB:
     wide_image += discretize_model(model, (0, IMAGE_SIZE), (0, IMAGE_SIZE),
                                    mode='oversample')
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_psf_photometry_discrete():
     """ Test psf_photometry with discrete PRF model. """
 
@@ -386,12 +440,13 @@ def test_psf_photometry_discrete():
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
                                     bkg_estimator=None, psf_model=prf,
                                     fitshape=7)
-    f = basic_phot(image=image, positions=INTAB)
+    f = basic_phot(image=image, init_guesses=INTAB)
 
     for n in ['x', 'y', 'flux']:
         assert_allclose(f[n + '_0'], f[n + '_fit'], rtol=1e-6)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_tune_coordinates():
     """
     Test psf_photometry with discrete PRF model and coordinates that need
@@ -406,14 +461,15 @@ def test_tune_coordinates():
     intab['x_0'] += 0.3
 
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                                bkg_estimator=None, psf_model=prf,
-                                fitshape=7)
+                                    bkg_estimator=None, psf_model=prf,
+                                    fitshape=7)
 
-    f = basic_phot(image=image, positions=intab)
+    f = basic_phot(image=image, init_guesses=intab)
     for n in ['x', 'y', 'flux']:
         assert_allclose(f[n + '_0'], f[n + '_fit'], rtol=1e-3)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_psf_boundary():
     """
     Test psf_photometry with discrete PRF model at the boundary of the data.
@@ -422,14 +478,15 @@ def test_psf_boundary():
     prf = DiscretePRF(test_psf, subsampling=1)
 
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                            bkg_estimator=None, psf_model=prf,
-                            fitshape=7, aperture_radius=5.5)
+                                    bkg_estimator=None, psf_model=prf,
+                                    fitshape=7, aperture_radius=5.5)
 
     intab = Table(data=[[1], [1]], names=['x_0', 'y_0'])
-    f = basic_phot(image=image, positions=intab)
+    f = basic_phot(image=image, init_guesses=intab)
     assert_allclose(f['flux_fit'], 0, atol=1e-8)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_aperture_radius_value_error():
     """
     Test psf_photometry with discrete PRF model at the boundary of the data.
@@ -438,16 +495,17 @@ def test_aperture_radius_value_error():
     prf = DiscretePRF(test_psf, subsampling=1)
 
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                            bkg_estimator=None, psf_model=prf,
-                            fitshape=7)
+                                    bkg_estimator=None, psf_model=prf,
+                                    fitshape=7)
 
     intab = Table(data=[[1], [1]], names=['x_0', 'y_0'])
     with pytest.raises(ValueError) as err:
-        f = basic_phot(image=image, positions=intab)
+        basic_phot(image=image, init_guesses=intab)
 
     assert 'aperture_radius is None' in str(err.value)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_psf_boundary_gaussian():
     """
     Test psf_photometry with discrete PRF model at the boundary of the data.
@@ -456,14 +514,15 @@ def test_psf_boundary_gaussian():
     psf = IntegratedGaussianPRF(GAUSSIAN_WIDTH)
 
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                            bkg_estimator=None, psf_model=psf,
-                            fitshape=7)
+                                    bkg_estimator=None, psf_model=psf,
+                                    fitshape=7)
 
     intab = Table(data=[[1], [1]], names=['x_0', 'y_0'])
-    f = basic_phot(image=image, positions=intab)
+    f = basic_phot(image=image, init_guesses=intab)
     assert_allclose(f['flux_fit'], 0, atol=1e-8)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_psf_photometry_gaussian():
     """
     Test psf_photometry with Gaussian PSF model.
@@ -472,13 +531,60 @@ def test_psf_photometry_gaussian():
     psf = IntegratedGaussianPRF(sigma=GAUSSIAN_WIDTH)
 
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                            bkg_estimator=None, psf_model=psf,
-                            fitshape=7)
-    f = basic_phot(image=image, positions=INTAB)
+                                    bkg_estimator=None, psf_model=psf,
+                                    fitshape=7)
+    f = basic_phot(image=image, init_guesses=INTAB)
     for n in ['x', 'y', 'flux']:
         assert_allclose(f[n + '_0'], f[n + '_fit'], rtol=1e-3)
 
-@pytest.mark.skipif('not HAS_SCIPY or not ASTROPY_GT_1_1')
+
+@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.parametrize("renormalize_psf", (True, False))
+def test_psf_photometry_gaussian2(renormalize_psf):
+    """
+    Test psf_photometry with Gaussian PSF model from Astropy.
+    """
+
+    psf = Gaussian2D(1. / (2 * np.pi * GAUSSIAN_WIDTH ** 2), PSF_SIZE // 2,
+                     PSF_SIZE // 2, GAUSSIAN_WIDTH, GAUSSIAN_WIDTH)
+    psf = prepare_psf_model(psf, xname='x_mean', yname='y_mean',
+                            renormalize_psf=renormalize_psf)
+
+    basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
+                                    bkg_estimator=None, psf_model=psf,
+                                    fitshape=7)
+    f = basic_phot(image=image, init_guesses=INTAB)
+
+    for n in ['x', 'y']:
+        assert_allclose(f[n + '_0'], f[n + '_fit'], rtol=1e-3)
+    # flux error worse, because of integration scheme ?
+    assert_allclose(f['flux_0'], f['flux_fit'], rtol=1)
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_psf_photometry_moffat():
+    """
+    Test psf_photometry with Moffat PSF model from Astropy.
+    """
+
+    psf = Moffat2D(1. / (2 * np.pi * GAUSSIAN_WIDTH ** 2), PSF_SIZE // 2,
+                   PSF_SIZE // 2, 1, 1)
+    psf = prepare_psf_model(psf, xname='x_0', yname='y_0',
+                            renormalize_psf=False)
+
+    basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
+                                    bkg_estimator=None, psf_model=psf,
+                                    fitshape=7)
+    f = basic_phot(image=image, init_guesses=INTAB)
+    f.pprint(max_width=-1)
+
+    for n in ['x', 'y']:
+        assert_allclose(f[n + '_0'], f[n + '_fit'], rtol=1e-3)
+    # image was created with a gaussian, so flux won't match exactly
+    assert_allclose(f['flux_0'], f['flux_fit'], rtol=1)
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
 def test_psf_fitting_data_on_edge():
     """
     No mask is input explicitly here, but source 2 is so close to the
@@ -488,10 +594,10 @@ def test_psf_fitting_data_on_edge():
     psf_guess = IntegratedGaussianPRF(flux=1, sigma=WIDE_GAUSSIAN_WIDTH)
     psf_guess.flux.fixed = psf_guess.x_0.fixed = psf_guess.y_0.fixed = False
     basic_phot = BasicPSFPhotometry(group_maker=DAOGroup(2),
-                            bkg_estimator=None, psf_model=psf_guess,
-                            fitshape=7)
+                                    bkg_estimator=None, psf_model=psf_guess,
+                                    fitshape=7)
 
-    outtab = basic_phot(image=wide_image, positions=WIDE_INTAB)
+    outtab = basic_phot(image=wide_image, init_guesses=WIDE_INTAB)
 
     for n in ['x', 'y', 'flux']:
         assert_allclose(outtab[n + '_0'], outtab[n + '_fit'],
