@@ -2,17 +2,15 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 from copy import deepcopy
-from distutils.version import LooseVersion
 
 import numpy as np
 from astropy.utils import lazyproperty
 
+from ..utils.colormaps import random_cmap
+
 
 __all__ = ['SegmentationImage']
 
-# outline_segments requires scikit-image >= 0.11
-__doctest_skip__ = {'SegmentationImage.outline_segments'}
-
 __doctest_requires__ = {('SegmentationImage', 'SegmentationImage.*'):
                         ['scipy', 'skimage']}
 
@@ -228,6 +226,35 @@ class SegmentationImage(object):
             raise ValueError('label "{0}" is not in the segmentation '
                              'image'.format(label))
 
+    def cmap(self, background_color='#000000', random_state=None):
+        """
+        A matplotlib colormap consisting of random (muted) colors.
+
+        This is very useful for plotting the segmentation image.
+
+        Parameters
+        ----------
+        background_color : str or `None`, optional
+            A hex string in the "#rrggbb" format defining the first
+            color in the colormap.  This color will be used as the
+            background color (label = 0) when plotting the segmentation
+            image.  The default is black.
+
+        random_state : int or `~numpy.random.RandomState`, optional
+            The pseudo-random number generator state used for random
+            sampling.  Separate function calls with the same
+            ``random_state`` will generate the same colormap.
+        """
+
+        from matplotlib import colors
+
+        cmap = random_cmap(self.max + 1, random_state=random_state)
+
+        if background_color is not None:
+            cmap.colors[0] = colors.hex2color(background_color)
+
+        return cmap
+
     def outline_segments(self, mask_background=False):
         """
         Outline the labeled segments.
@@ -270,10 +297,7 @@ class SegmentationImage(object):
                [0, 0, 0, 0, 0, 0]])
         """
 
-        import skimage
-        if LooseVersion(skimage.__version__) < LooseVersion('0.11'):
-            raise ImportError('The outline_segments() function requires '
-                              'scikit-image >= 0.11')
+        # requires scikit-image >= 0.11
         from skimage.segmentation import find_boundaries
 
         outlines = self.data * find_boundaries(self.data, mode='inner')
