@@ -10,7 +10,7 @@ from .check_random_state import check_random_state
 __all__ = ['random_cmap']
 
 
-def random_cmap(ncolors=256, background_color='black', random_state=None):
+def random_cmap(ncolors=256, random_state=None):
     """
     Generate a matplotlib colormap consisting of random (muted) colors.
 
@@ -19,14 +19,7 @@ def random_cmap(ncolors=256, background_color='black', random_state=None):
     Parameters
     ----------
     ncolors : int, optional
-        The number of colors in the colormap.  For use with segmentation
-        images, ``ncolors`` should be set to the number of labels.  The
-        default is 256.
-
-    background_color : str, optional
-        The name of the background (first) color in the colormap.  Valid
-        colors names are defined by ``matplotlib.colors.cnames``.  The
-        default is ``'black'``.
+        The number of colors in the colormap.  The default is 256.
 
     random_state : int or `~numpy.random.RandomState`, optional
         The pseudo-random number generator state used for random
@@ -48,10 +41,4 @@ def random_cmap(ncolors=256, background_color='black', random_state=None):
     hsv = np.dstack((h, s, v))
     rgb = np.squeeze(colors.hsv_to_rgb(hsv))
 
-    if background_color is not None:
-        if background_color not in colors.cnames:
-            raise ValueError('"{0}" is not a valid background color '
-                             'name'.format(background_color))
-        rgb[0] = colors.hex2color(colors.cnames[background_color])
-
     return colors.ListedColormap(rgb)
