@@ -1,8 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
-import numpy as np
 import warnings
+
+import numpy as np
+from astropy.utils.decorators import deprecated
 from astropy.utils.exceptions import AstropyUserWarning
 
 
@@ -154,7 +156,6 @@ class ShepardIDWInterpolator(object):
 
     def __call__(self, positions, n_neighbors=8, eps=0.0, power=1.0, reg=0.0,
                  conf_dist=1e-12, dtype=np.float):
-
         """
         Evaluate the interpolator at the given positions.
 
@@ -285,6 +286,7 @@ class ShepardIDWInterpolator(object):
             return interp_values
 
 
+@deprecated('0.4')
 def interpolate_masked_data(data, mask, error=None, background=None):
     """
     Interpolate over masked pixels in data and optional error or
@@ -445,4 +447,5 @@ def mask_to_mirrored_num(image, mask_image, center_position, bbox=None):
 
     outimage = np.copy(image)
     outimage[bbox[2]:bbox[3]+1, bbox[0]:bbox[1]+1] = subdata
+
     return outimage
