@@ -3,7 +3,7 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import numpy as np
-from astropy.tests.helper import pytest
+import pytest
 
 from .. import check_random_state
 
